@@ -31,10 +31,12 @@
 
 #include "atacmds.h"
 #include "config.h"
+#include "int64.h"
 #include "extern.h"
 #include "utility.h"
 
-const char *atacmds_c_cvsid="$Id: atacmds.c,v 1.139 2004/02/09 19:31:06 ballen4705 Exp $" ATACMDS_H_CVSID CONFIG_H_CVSID EXTERN_H_CVSID UTILITY_H_CVSID;
+const char *atacmds_c_cvsid="$Id: atacmds.c,v 1.153 2004/06/03 03:47:06 ballen4705 Exp $"
+ATACMDS_H_CVSID CONFIG_H_CVSID EXTERN_H_CVSID INT64_H_CVSID UTILITY_H_CVSID;
 
 // to hold onto exit code for atexit routine
 extern int exitstatus;
@@ -173,6 +175,32 @@ const char *vendorattributeargs[] = {
   NULL
 };
 
+// This are the meanings of the Self-test failure checkpoint byte.
+// This is in the self-test log at offset 4 bytes into the self-test
+// descriptor and in the SMART READ DATA structure at byte offset
+// 371. These codes are not well documented.  The meanings returned by
+// this routine are used (at least) by Maxtor and IBM. Returns NULL if
+// not recognized.  Currently the maximum length is 15 bytes.
+const char *SelfTestFailureCodeName(unsigned char which){
+  
+  switch (which) {
+  case 0:
+    return "Write_Test";
+  case 1:
+    return "Servo_Basic";
+  case 2:
+    return "Servo_Random";
+  case 3:
+    return "G-list_Scan";
+  case 4:
+    return "Handling_Damage";
+  case 5:
+    return "Read_Scan";
+  default:
+    return NULL;
+  }
+}
+
 // This is a utility function for parsing pairs like "9,minutes" or
 // "220,temp", and putting the correct flag into the attributedefs
 // array.  Returns 1 if problem, 0 if pair has been recongized.
@@ -442,7 +470,7 @@ char *create_vendor_attribute_arg_list(void){
       strcat(s, "\n");
   }
 
-  free(sorted);
+  free((char **)sorted);
 
   // Return a pointer to the string
   return s;
@@ -491,6 +519,7 @@ static char *commandstrings[]={
   "IDENTIFY DEVICE",
   "IDENTIFY PACKET DEVICE",
   "CHECK POWER MODE",
+  "SMART WRITE LOG",
   "WARNING (UNDEFINED COMMAND -- CONTACT DEVELOPERS AT " PACKAGE_BUGREPORT ")\n"
 };
 
@@ -519,6 +548,8 @@ int smartcommandhandler(int device, smart_command_set command, int select, char
                 command==READ_THRESHOLDS || 
                 command==READ_VALUES ||
 		command==CHECK_POWER_MODE);
+
+  int sendsdata=(command==WRITE_LOG);
   
   // If reporting is enabled, say what the command will be before it's executed
   if (con->reportataioctl){
@@ -526,7 +557,8 @@ int smartcommandhandler(int device, smart_command_set command, int select, char
           int usesparam=(command==READ_LOG || 
                          command==AUTO_OFFLINE || 
                          command==AUTOSAVE || 
-                         command==IMMEDIATE_OFFLINE);
+                         command==IMMEDIATE_OFFLINE ||
+                         command==WRITE_LOG);
                   
     pout("\nREPORT-IOCTL: DeviceFD=%d Command=%s", device, commandstrings[command]);
     if (usesparam)
@@ -535,7 +567,7 @@ int smartcommandhandler(int device, smart_command_set command, int select, char
       pout("\n");
   }
   
-  if (getsdata && !data){
+  if ((getsdata || sendsdata) && !data){
     pout("REPORT-IOCTL: Unable to execute command %s : data destination address is NULL\n", commandstrings[command]);
     return -1;
   }
@@ -550,6 +582,15 @@ int smartcommandhandler(int device, smart_command_set command, int select, char
       memset(data, '\0', 512);
   }
 
+
+  // If reporting is enabled, say what input was sent to the command
+  if (con->reportataioctl && sendsdata){
+    pout("REPORT-IOCTL: DeviceFD=%d Command=%s", device, commandstrings[command]);
+    // if requested, pretty-print the output data structure
+    if (con->reportataioctl>1)
+      prettyprint((unsigned char *)data, commandstrings[command]);
+  }
+
   // In case the command produces an error, we'll want to know what it is:
   errno=0;
   
@@ -842,9 +883,14 @@ int ataReadSelectiveSelfTestLog(int device, struct ata_selective_self_test_log *
   
   // get data from device
   if (smartcommandhandler(device, READ_LOG, 0x09, (char *)data)){
+    syserror("Error SMART Read Selective Self-Test Log failed");
     return -1;
   }
-
+   
+  // compute its checksum, and issue a warning if needed
+  if (checksum((unsigned char *)data))
+    checksumwarning("SMART Selective Self-Test Log Structure");
+  
   // swap endian order if needed
   if (isbigendian()){
     int i;
@@ -858,35 +904,72 @@ int ataReadSelectiveSelfTestLog(int device, struct ata_selective_self_test_log *
     swap2((char *)&(data->flags));
     swap2((char *)&(data->pendingtime));
   }
+  
+  if (data->logversion != 1)
+    pout("SMART Selective Self-Test Log Data Structure Revision Number (%d) should be 1\n", data->logversion);
+  
   return 0;
 }
 
-#if DEVELOP_SELECTIVE_SELF_TEST
 // Writes the selective self-test log (log #9)
-int ataWriteSelectiveSelfTestLog(int device){   
+int ataWriteSelectiveSelfTestLog(int device, struct ata_smart_values *sv){   
   int i;
   struct ata_selective_self_test_log sstlog, *data=&sstlog;
   unsigned char cksum=0;
   unsigned char *ptr=(unsigned char *)data;
-
+  
   // Read log
-  ataReadSelectiveSelfTestLog(device, data);
+  if (ataReadSelectiveSelfTestLog(device, data)) {
+    pout("Since Read failed, will not attempt to WRITE Selective Self-test Log\n");
+    return -1;
+  }
+  
+  // Fix logversion if needed
+  if (data->logversion !=1) {
+    pout("Error SMART Selective Self-Test Log Data Structure Revision not recognized\n"
+	 "Revision number should be 1 but is %d.  To be safe, aborting WRITE LOG\n", data->logversion);
+    return -2;
+  }
 
+  // Host is NOT allowed to write selective self-test log if a selective
+  // self-test is in progress.
+  if (0<data->currentspan && data->currentspan<6 && ((sv->self_test_exec_status)>>4)==15) {
+    pout("Error SMART Selective or other Self-Test in progress.\n");
+    return -4;
+  }
+  
   // Clear spans
   for (i=0; i<5; i++)
     memset(data->span+i, 0, sizeof(struct test_span));
-
+  
   // Set spans for testing 
   for (i=0; i<con->smartselectivenumspans; i++){
     data->span[i].start = con->smartselectivespan[i][0];
     data->span[i].end   = con->smartselectivespan[i][1];
   }
 
-  // Do NOT perform off-line scan after selective 
-  data->flags=0;
-  data->undefined=data->checksum=0;
+  // host must initialize to zero before initiating selective self-test
+  data->currentlba=0;
+  data->currentspan=0;
+  
+  // Perform off-line scan after selective test?
+  if (1 == con->scanafterselect)
+    // NO
+    data->flags &= ~SELECTIVE_FLAG_DOSCAN;
+  else if (2 == con->scanafterselect)
+    // YES
+    data->flags |= SELECTIVE_FLAG_DOSCAN;
+  
+  // Must clear active and pending flags before writing
+  data->flags &= ~(SELECTIVE_FLAG_ACTIVE);  
+  data->flags &= ~(SELECTIVE_FLAG_PENDING);
 
-  // Put in correct checksum
+  // modify pending time?
+  if (con->pendingtime)
+    data->pendingtime=(unsigned short)(con->pendingtime-1);
+
+  // Set checksum to zero, then compute checksum
+  data->checksum=0;
   for (i=0; i<512; i++)
     cksum+=ptr[i];
   cksum=~cksum;
@@ -907,14 +990,14 @@ int ataWriteSelectiveSelfTestLog(int device){
     swap2((char *)&(data->pendingtime));
   }
 
-  // send data to device -- ioctl NOT implemented yet!
+  // write new selective self-test log
   if (smartcommandhandler(device, WRITE_LOG, 0x09, (char *)data)){
-    return -1;
+    syserror("Error Write Selective Self-Test Log failed");
+    return -3;
   }
 
   return 0;
 }
-#endif
 
 // This corrects some quantities that are byte reversed in the SMART
 // ATA ERROR LOG.
@@ -1078,29 +1161,17 @@ int ataDoesSmartWork(int device){
 // This function uses a different interface (DRIVE_TASK) than the
 // other commands in this file.
 int ataSmartStatus2(int device){
-
-  int returnval=smartcommandhandler(device, STATUS_CHECK, 0, NULL);
-  
-  if (returnval==-1){
-    syserror("Error SMART Status command via HDIO_DRIVE_TASK failed");
-    pout("Rebuild older linux 2.2 kernels with HDIO_DRIVE_TASK support enabled\n");
-  }
-  
-  return returnval;
+  return smartcommandhandler(device, STATUS_CHECK, 0, NULL);  
 }
 
 // This is the way to execute ALL tests: offline, short self-test,
 // extended self test, with and without captive mode, etc.
-int ataSmartTest(int device, int testtype){     
+int ataSmartTest(int device, int testtype, struct ata_smart_values *sv) {     
   char cmdmsg[128],*type,*captive;
-  int errornum;
-  int cap,select=0;
+  int errornum, cap, retval, select=0;
 
   // Boolean, if set, says test is captive
   cap=testtype & CAPTIVE_MASK;
-  
-  // Boolean, if set, then test is selective
-  select=(testtype==SELECTIVE_SELF_TEST || testtype==SELECTIVE_CAPTIVE_SELF_TEST);
 
   // Set up strings that describe the type of test
   if (cap)
@@ -1116,20 +1187,19 @@ int ataSmartTest(int device, int testtype){
     type="Extended self-test";
   else if (testtype==CONVEYANCE_SELF_TEST || testtype==CONVEYANCE_CAPTIVE_SELF_TEST)
     type="Conveyance self-test";
-#if DEVELOP_SELECTIVE_SELF_TEST
-  else if (select){
-    int i;
+  else if ((select=(testtype==SELECTIVE_SELF_TEST || testtype==SELECTIVE_CAPTIVE_SELF_TEST)))
     type="Selective self-test";
-    pout("Using test spans:\n");
-    for (i = 0; i < con->smartselectivenumspans; i++)
-      pout("%lld - %lld\n",
-           con->smartselectivespan[i][0],
-           con->smartselectivespan[i][1]);
-  }
-#endif
   else
     type="[Unrecognized] self-test";
   
+  // If doing a selective self-test, first use WRITE_LOG to write the
+  // selective self-test log.
+  if (select && (retval=ataWriteSelectiveSelfTestLog(device, sv))) {
+    if (retval==-4)
+      pout("Can't start selective self-test without aborting current test: use '-X' option to smartctl.\n");
+    return retval;
+  }
+
   //  Print ouf message that we are sending the command to test
   if (testtype==ABORT_SELF_TEST)
     sprintf(cmdmsg,"Abort SMART off-line mode self-test routine");
@@ -1137,20 +1207,18 @@ int ataSmartTest(int device, int testtype){
     sprintf(cmdmsg,"Execute SMART %s routine immediately in %s mode",type,captive);
   pout("Sending command: \"%s\".\n",cmdmsg);
 
-#if DEVELOP_SELECTIVE_SELF_TEST
-  // If doing a selective self-test, issue WRITE_LOG command
-  if (select && ataWriteSelectiveSelfTestLog(device)){
-    char errormsg[256];
-    sprintf(errormsg,"Command \"%s\" failed to WRITE Selective Self-Test Log ",cmdmsg); 
-    syserror(errormsg);
-    pout("\n");
-    return -1;
+  if (select) {
+    int i;
+    pout("SPAN         STARTING_LBA           ENDING_LBA\n");
+    for (i = 0; i < con->smartselectivenumspans; i++)
+      pout("   %d %20"PRId64" %20"PRId64"\n", i,
+           con->smartselectivespan[i][0],
+           con->smartselectivespan[i][1]);
   }
-#endif
-
+  
   // Now send the command to test
   errornum=smartcommandhandler(device, IMMEDIATE_OFFLINE, testtype, NULL);
-
+  
   if (errornum && !(cap && errno==EIO)){
     char errormsg[128];
     sprintf(errormsg,"Command \"%s\" failed",cmdmsg); 
@@ -1375,10 +1443,10 @@ int ataCheckAttribute(struct ata_smart_values *data,
 // array defs[] contains non-zero values if particular attributes have
 // non-default interpretations.
 
-long long ataPrintSmartAttribRawValue(char *out, 
-                                      struct ata_smart_attribute *attribute,
-                                      unsigned char *defs){
-  long long rawvalue;
+int64_t ataPrintSmartAttribRawValue(char *out, 
+                                    struct ata_smart_attribute *attribute,
+                                    unsigned char *defs){
+  int64_t rawvalue;
   unsigned word[3];
   int j;
   unsigned char select;
@@ -1391,7 +1459,7 @@ long long ataPrintSmartAttribRawValue(char *out,
     // succumb to the temptation to use raw[j]<<(8*j) since under
     // the normal rules this will be promoted to the native type.
     // On a 32 bit machine this might then overflow.
-    long long temp;
+    int64_t temp;
     temp = attribute->raw[j];
     temp <<= 8*j;
     rawvalue |= temp;
@@ -1426,7 +1494,7 @@ long long ataPrintSmartAttribRawValue(char *out,
   
   // Print one six-byte quantity
   if (select==255){
-    out+=sprintf(out, "%llu", rawvalue);
+    out+=sprintf(out, "%"PRIu64, rawvalue);
     return rawvalue;
   }
 
@@ -1444,26 +1512,26 @@ long long ataPrintSmartAttribRawValue(char *out,
   case 9:
     if (select==1){
       // minutes
-      long long tmp1=rawvalue/60;
-      long long tmp2=rawvalue%60;
-      out+=sprintf(out, "%lluh+%02llum", tmp1, tmp2);
+      int64_t tmp1=rawvalue/60;
+      int64_t tmp2=rawvalue%60;
+      out+=sprintf(out, "%"PRIu64"h+%02"PRIu64"m", tmp1, tmp2);
     }
     else if (select==3){
       // seconds
-      long long hours=rawvalue/3600;
-      long long minutes=(rawvalue-3600*hours)/60;
-      long long seconds=rawvalue%60;
-      out+=sprintf(out, "%lluh+%02llum+%02llus", hours, minutes, seconds);
+      int64_t hours=rawvalue/3600;
+      int64_t minutes=(rawvalue-3600*hours)/60;
+      int64_t seconds=rawvalue%60;
+      out+=sprintf(out, "%"PRIu64"h+%02"PRIu64"m+%02"PRIu64"s", hours, minutes, seconds);
     }
     else if (select==4){
       // 30-second counter
-      long long tmp1=rawvalue/120;
-      long long tmp2=(rawvalue-120*tmp1)/2;
-      out+=sprintf(out, "%lluh+%02llum", tmp1, tmp2);
+      int64_t tmp1=rawvalue/120;
+      int64_t tmp2=(rawvalue-120*tmp1)/2;
+      out+=sprintf(out, "%"PRIu64"h+%02"PRIu64"m", tmp1, tmp2);
     }
     else
       // hours
-      out+=sprintf(out, "%llu", rawvalue);  //stored in hours
+      out+=sprintf(out, "%"PRIu64, rawvalue);  //stored in hours
     break;
    // Load unload cycles
   case 193:
@@ -1475,7 +1543,7 @@ long long ataPrintSmartAttribRawValue(char *out,
     }
     else
       // associated
-      out+=sprintf(out, "%llu", rawvalue);
+      out+=sprintf(out, "%"PRIu64, rawvalue);
     break;
     // Temperature
   case 194:
@@ -1487,7 +1555,7 @@ long long ataPrintSmartAttribRawValue(char *out,
     }
     else if (select==2)
       // unknown attribute
-      out+=sprintf(out, "%llu", rawvalue);
+      out+=sprintf(out, "%"PRIu64, rawvalue);
     else {
       out+=sprintf(out, "%d", word[0]);
       if (!(rawvalue==word[0]))
@@ -1496,7 +1564,7 @@ long long ataPrintSmartAttribRawValue(char *out,
     }
     break;
   default:
-    out+=sprintf(out, "%llu", rawvalue);
+    out+=sprintf(out, "%"PRIu64, rawvalue);
   }
   
   // Return the full value
@@ -1744,3 +1812,41 @@ void ataPrintSmartAttribName(char *out, unsigned char id, unsigned char *definit
   sprintf(out,"%3hu %s",(short int)id,name);
   return;
 }
+
+// Returns raw value of Attribute with ID==id. This will be in the
+// range 0 to 2^48-1 inclusive.  If the Attribute does not exist,
+// return -1.
+int64_t ATAReturnAttributeRawValue(unsigned char id, struct ata_smart_values *data) {
+  int i;
+
+  // valid Attribute IDs are in the range 1 to 255 inclusive.
+  if (!id || !data)
+    return -1;
+  
+  // loop over Attributes to see if there is one with the desired ID
+  for (i=0; i<NUMBER_ATA_SMART_ATTRIBUTES; i++) {
+    struct ata_smart_attribute *this = data->vendor_attributes + i;
+    if (this->id == id) {
+      // we've found the desired Attribute.  Return its value
+      int64_t rawvalue=0;
+      int j;
+
+      for (j=0; j<6; j++) {
+	// This looks a bit roundabout, but is necessary.  Don't
+	// succumb to the temptation to use raw[j]<<(8*j) since under
+	// the normal rules this will be promoted to the native type.
+	// On a 32 bit machine this might then overflow.
+	int64_t temp;
+	temp = this->raw[j];
+	temp <<= 8*j;
+	rawvalue |= temp;
+      } // loop over j
+      return rawvalue;
+    } // found desired Attribute
+  } // loop over Attributes
+  
+  // fall-through: no such Attribute found
+  return -1;
+}
+
+
