@@ -25,12 +25,9 @@
 #ifndef ATACMDS_H_
 #define ATACMDS_H_
 
-#define ATACMDS_H_CVSID "$Id: atacmds.h,v 1.63 2004/02/04 14:21:49 ballen4705 Exp $\n"
-
-#include <sys/ioctl.h>
-#include <sys/fcntl.h>
-#include <sys/types.h>
+#define ATACMDS_H_CVSID "$Id: atacmds.h,v 1.69 2004/04/17 10:44:47 ballen4705 Exp $\n"
 
+#include "int64.h"
 
 typedef enum {
   // returns no data, just succeeds or fails
@@ -48,11 +45,9 @@ typedef enum {
   IDENTIFY,
   PIDENTIFY,
   // returns 1 byte of data
-  CHECK_POWER_MODE
-#if DEVELOP_SELECTIVE_SELF_TEST
+  CHECK_POWER_MODE,
   // writes 512 bytes of data:
-  , WRITE_LOG
-#endif
+  WRITE_LOG
 } smart_command_set;
 
 // ATA Specification Command Register Values (Commands)
@@ -156,13 +151,13 @@ struct ata_smart_attribute {
 // both normal operation and off-line testing.
 #define ATTRIBUTE_FLAGS_ONLINE(x) (x & 0x02)
 
-// 2: Performance bit
 
-// The following are (probably) IBM's or Quantum's definitions for the
+// The following are (probably) IBM's, Maxtors and  Quantum's definitions for the
 // vendor-specific bits:
+// 2: Performance type bit
 #define ATTRIBUTE_FLAGS_PERFORMANCE(x) (x & 0x04)
 
-// 3: Errorrate bit
+// 3: Errorrate type bit
 #define ATTRIBUTE_FLAGS_ERRORRATE(x) (x & 0x08)
 
 // 4: Eventcount bit
@@ -183,20 +178,29 @@ struct ata_smart_values {
   unsigned char offline_data_collection_status;
   unsigned char self_test_exec_status;  //IBM # segments for offline collection
   unsigned short int total_time_to_complete_off_line; // IBM different
-  unsigned char vendor_specific_366; // IBM curent segment pointer
+  unsigned char vendor_specific_366; // Maxtor & IBM curent segment pointer
   unsigned char offline_data_collection_capability;
   unsigned short int smart_capability;
   unsigned char errorlog_capability;
-  unsigned char vendor_specific_371;  // IBM: self-test failure checkpoint
+  unsigned char vendor_specific_371;  // Maxtor, IBM: self-test failure checkpoint see below!
   unsigned char short_test_completion_time;
   unsigned char extend_test_completion_time;
   unsigned char conveyance_test_completion_time;
   unsigned char reserved_375_385[11];
-  unsigned char vendor_specific_386_510[125];
+  unsigned char vendor_specific_386_510[125]; // Maxtor bytes 508-509 Attribute/Threshold Revision #
   unsigned char chksum;
 }; 
 #pragma pack()
 
+/* Maxtor, IBM: self-test failure checkpoint byte meaning:
+ 00 - write test
+ 01 - servo basic
+ 02 - servo random
+ 03 - G-list scan
+ 04 - Handling damage
+ 05 - Read scan
+*/
+
 /* Vendor attribute of SMART Threshold (compare to ata_smart_attribute above) */
 #pragma pack(1)
 struct ata_smart_threshold_entry {
@@ -314,8 +318,8 @@ struct ata_smart_log_directory {
 // Revision 3
 #pragma pack(1)
 struct test_span {
-  unsigned long long start;
-  unsigned long long end;
+  uint64_t start;
+  uint64_t end;
 };
 #pragma pack()
 
@@ -323,18 +327,22 @@ struct test_span {
 struct ata_selective_self_test_log {
   unsigned short     logversion;
   struct test_span   span[5];
-  unsigned char      reserved[337-82+1];
+  unsigned char      reserved1[337-82+1];
   unsigned char      vendor_specific1[491-338+1];
-  unsigned long long currentlba;
+  uint64_t           currentlba;
   unsigned short     currentspan;
   unsigned short     flags;
   unsigned char      vendor_specific2[507-504+1];
   unsigned short     pendingtime;
-  unsigned char      undefined;  // Error in specs!
+  unsigned char      reserved2;
   unsigned char      checksum;
 };
 #pragma pack()
 
+#define SELECTIVE_FLAG_DOSCAN  (0x0002)
+#define SELECTIVE_FLAG_PENDING (0x0008)
+#define SELECTIVE_FLAG_ACTIVE  (0x0010)
+
 // Get information from drive
 int ataReadHDIdentity(int device, struct ata_identify_device *buf);
 int ataCheckPowerMode(int device);
@@ -344,6 +352,7 @@ int ataReadSmartValues(int device,struct ata_smart_values *);
 int ataReadSmartThresholds(int device, struct ata_smart_thresholds_pvt *);
 int ataReadErrorLog(int device, struct ata_smart_errorlog *);
 int ataReadSelfTestLog(int device, struct ata_smart_selftestlog *);
+int ataReadSelectiveSelfTestLog(int device, struct ata_selective_self_test_log *data);
 int ataSmartStatus(int device);
 int ataSetSmartThresholds(int device, struct ata_smart_thresholds_pvt *);
 int ataReadLogDirectory(int device, struct ata_smart_log_directory *);  
@@ -417,15 +426,15 @@ int isSupportConveyanceSelfTest(struct ata_smart_values *data);
 
 int isSupportSelectiveSelfTest(struct ata_smart_values *data);
 
-int ataSmartTest(int device, int testtype);
+int ataSmartTest(int device, int testtype, struct ata_smart_values *data);
 
 int TestTime(struct ata_smart_values *data,int testtype);
 
 // Prints the raw value (with appropriate formatting) into the
 // character string out.
-long long ataPrintSmartAttribRawValue(char *out, 
-                                      struct ata_smart_attribute *attribute,
-                                      unsigned char *defs);
+int64_t ataPrintSmartAttribRawValue(char *out, 
+                                    struct ata_smart_attribute *attribute,
+                                    unsigned char *defs);
 
 // Prints Attribute Name for standard SMART attributes. Writes a
 // 30 byte string with attribute name into output
@@ -447,6 +456,21 @@ int ataCheckAttribute(struct ata_smart_values *data,
 // Structure with the incorrect checksum.
 void checksumwarning(const char *string);
 
+// Returns raw value of Attribute with ID==id. This will be in the
+// range 0 to 2^48-1 inclusive.  If the Attribute does not exist,
+// return -1.
+int64_t ATAReturnAttributeRawValue(unsigned char id, struct ata_smart_values *data);
+
+
+// This are the meanings of the Self-test failure checkpoint byte.
+// This is in the self-test log at offset 4 bytes into the self-test
+// descriptor and in the SMART READ DATA structure at byte offset
+// 371. These codes are not well documented.  The meanings returned by
+// this routine are used (at least) by Maxtor and IBM. Returns NULL if
+// not recognized.
+const char *SelfTestFailureCodeName(unsigned char which);
+
+
 #define MAX_ATTRIBUTE_NUM 256
 
 extern const char *vendorattributeargs[];
