@@ -24,19 +24,19 @@
 
 #include <ctype.h>
 #include <stdio.h>
-#include <syslog.h>
 #include <string.h>
 #include "atacmdnames.h"
 #include "atacmds.h"
 #include "ataprint.h"
 #include "smartctl.h"
+#include "int64.h"
 #include "extern.h"
 #include "utility.h"
 #include "knowndrives.h"
 #include "config.h"
 
-const char *ataprint_c_cvsid="$Id: ataprint.c,v 1.140 2004/03/06 22:08:00 ballen4705 Exp $"
-ATACMDNAMES_H_CVSID ATACMDS_H_CVSID ATAPRINT_H_CVSID CONFIG_H_CVSID EXTERN_H_CVSID KNOWNDRIVES_H_CVSID SMARTCTL_H_CVSID UTILITY_H_CVSID;
+const char *ataprint_c_cvsid="$Id: ataprint.c,v 1.154 2004/06/04 20:59:16 ballen4705 Exp $"
+ATACMDNAMES_H_CVSID ATACMDS_H_CVSID ATAPRINT_H_CVSID CONFIG_H_CVSID EXTERN_H_CVSID INT64_H_CVSID KNOWNDRIVES_H_CVSID SMARTCTL_H_CVSID UTILITY_H_CVSID;
 
 // for passing global control variables
 extern smartmonctrl *con;
@@ -426,7 +426,7 @@ char *construct_st_er_desc(struct ata_smart_errorlog_struct *data) {
 void ataPrintDriveInfo (struct ata_identify_device *drive){
   int version, drivetype;
   const char *description;
-  char unknown[64], timedatetz[64];
+  char unknown[64], timedatetz[DATEANDEPOCHLEN];
   unsigned short minorrev;
   char model[64], serial[64], firm[64];
   
@@ -486,44 +486,44 @@ void ataPrintDriveInfo (struct ata_identify_device *drive){
 }
 
 
-/*  prints verbose value Off-line data collection status byte */
-void PrintSmartOfflineStatus(struct ata_smart_values *data){
-  char *message=NULL;
-
-  // the final 7 bits
-  unsigned char stat=data->offline_data_collection_status & 0x7f;
+const char *OfflineDataCollectionStatus(unsigned char status_byte){
+  unsigned char stat=status_byte & 0x7f;
+  
+  switch(stat){
+  case 0x00:
+    return "was never started";
+  case 0x02:
+    return "was completed without error";
+  case 0x03:
+    if (status_byte == 0x03)
+      return "is in progress";
+    else
+      return "is in a Reserved state";
+  case 0x04:
+    return "was suspended by an interrupting command from host";
+  case 0x05:
+    return "was aborted by an interrupting command from host";
+  case 0x06:
+    return "was aborted by the device with a fatal error";
+  default:
+    if (stat >= 0x40)
+      return "is in a Vendor Specific state\n";
+    else
+      return "is in a Reserved state\n";
+  }
+}
+  
+  
+  /*  prints verbose value Off-line data collection status byte */
+  void PrintSmartOfflineStatus(struct ata_smart_values *data){
   
   pout("Offline data collection status:  (0x%02x)\t",
        (int)data->offline_data_collection_status);
     
-  switch(stat){
-  case 0x00:
-    message="never started";
-    break;
-  case 0x02:
-    message="completed without error";
-    break;
-  case 0x04:
-    message="suspended by an interrupting command from host";
-    break;
-  case 0x05:
-    message="aborted by an interrupting command from host";
-    break;
-  case 0x06:
-    message="aborted by the device with a fatal error";
-    break;
-  default:
-    if (stat >= 0x40)
-      pout("Vendor Specific.\n");
-    else
-      pout("Reserved.\n");
-  }
-  
-  if (message)
-    // Off-line data collection status byte is not a reserved
-    // or vendor specific value
-    pout("Offline data collection activity was\n"
-         "\t\t\t\t\t%s.\n", message);
+  // Off-line data collection status byte is not a reserved
+  // or vendor specific value
+  pout("Offline data collection activity\n"
+       "\t\t\t\t\t%s.\n", OfflineDataCollectionStatus(data->offline_data_collection_status));
   
   // Report on Automatic Data Collection Status.  Only IBM documents
   // this bit.  See SFF 8035i Revision 2 for details.
@@ -924,8 +924,9 @@ int ataPrintSmartErrorlog(struct ata_smart_errorlog *data){
        "\tDC = Device Command Register [HEX]\n"
        "\tER = Error register [HEX]\n"
        "\tST = Status register [HEX]\n"
-       "Timestamp = decimal seconds since the previous disk power-on.\n"
-       "Note: timestamp \"wraps\" after 2^32 msec = 49.710 days.\n\n");
+       "Powered_Up_Time is measured from power on, and printed as\n"
+       "DDd+hh:mm:SS.sss where DD=days, hh=hours, mm=minutes,\n"
+       "SS=sec, and sss=millisec. It \"wraps\" after 49.710 days.\n\n");
   
   // now step through the five error log data structures (table 39 of spec)
   for (k = 4; k >= 0; k-- ) {
@@ -941,6 +942,8 @@ int ataPrintSmartErrorlog(struct ata_smart_errorlog *data){
       // Table 57 of T13/1532D Volume 1 Revision 3
       char *msgstate;
       int bits=summary->state & 0x0f;
+      int days = (int)summary->timestamp/24;
+
       switch (bits){
       case 0x00: msgstate="in an unknown state";break;
       case 0x01: msgstate="sleeping"; break;
@@ -956,8 +959,8 @@ int ataPrintSmartErrorlog(struct ata_smart_errorlog *data){
 
       // See table 42 of ATA5 spec
       PRINT_ON(con);
-      pout("Error %d occurred at disk power-on lifetime: %d hours\n",
-             (int)(data->ata_error_count+k-4), (int)summary->timestamp);
+      pout("Error %d occurred at disk power-on lifetime: %d hours (%d days + %d hours)\n",
+             (int)(data->ata_error_count+k-4), (int)summary->timestamp, days, (int)(summary->timestamp-24*days));
       PRINT_OFF(con);
       pout("  When the command that caused the error occurred, the device was %s.\n\n",msgstate);
       pout("  After command completion occurred, registers were:\n"
@@ -980,14 +983,19 @@ int ataPrintSmartErrorlog(struct ata_smart_errorlog *data){
       }
       pout("\n\n");
       pout("  Commands leading to the command that caused the error were:\n"
-           "  CR FR SC SN CL CH DH DC   Timestamp  Command/Feature_Name\n"
-           "  -- -- -- -- -- -- -- --   ---------  --------------------\n");
+           "  CR FR SC SN CL CH DH DC   Powered_Up_Time  Command/Feature_Name\n"
+           "  -- -- -- -- -- -- -- --  ----------------  --------------------\n");
       for ( j = 4; j >= 0; j--){
         struct ata_smart_errorlog_command_struct *thiscommand=elog->commands+j;
-        
+
         // Spec says: unused data command structures shall be zero filled
-        if (nonempty((unsigned char*)thiscommand,sizeof(*thiscommand)))
-          pout("  %02x %02x %02x %02x %02x %02x %02x %02x %7d.%03d  %s\n",
+        if (nonempty((unsigned char*)thiscommand,sizeof(*thiscommand))) {
+	  char timestring[32];
+	  
+	  // Convert integer milliseconds to a text-format string
+	  MsecToText(thiscommand->timestamp, timestring);
+	  
+          pout("  %02x %02x %02x %02x %02x %02x %02x %02x  %16s  %s\n",
                (int)thiscommand->commandreg,
                (int)thiscommand->featuresreg,
                (int)thiscommand->sector_count,
@@ -996,9 +1004,9 @@ int ataPrintSmartErrorlog(struct ata_smart_errorlog *data){
                (int)thiscommand->cylinder_high,
                (int)thiscommand->drive_head,
                (int)thiscommand->devicecontrolreg,
-               (unsigned int)(thiscommand->timestamp / 1000U),
-               (unsigned int)(thiscommand->timestamp % 1000U),
+	       timestring,
                look_up_ata_command(thiscommand->commandreg, thiscommand->featuresreg));
+	}
       }
       pout("\n");
     }
@@ -1010,6 +1018,123 @@ int ataPrintSmartErrorlog(struct ata_smart_errorlog *data){
   return data->ata_error_count;  
 }
 
+void ataPrintSelectiveSelfTestLog(struct ata_selective_self_test_log *log, struct ata_smart_values *sv) {
+  int i,field1,field2;
+  char *msg;
+  char tmp[64];
+  uint64_t maxl=0,maxr=0;
+  uint64_t current=log->currentlba;
+  uint64_t currentend=current+65535;
+
+  // print data structure revision number
+  pout("SMART Selective self-test log data structure revision number %d\n",(int)log->logversion);
+  if (1 != log->logversion)
+    pout("Warning: ATA Specification requires selective self-test log data structure revision number = 1\n");
+  
+  switch((sv->self_test_exec_status)>>4){
+  case  0:msg="Completed";
+    break;
+  case  1:msg="Aborted_by_host";
+    break;
+  case  2:msg="Interrupted";
+    break;
+  case  3:msg="Fatal_error";
+    break;
+  case  4:msg="Completed_unknown_failure";
+    break;
+  case  5:msg="Completed_electrical_failure";
+    break;
+  case  6:msg="Completed_servo/seek_failure";
+    break;
+  case  7:msg="Completed_read_failure";
+    break;
+  case  8:msg="Completed_handling_damage??";
+    break;
+  case 15:msg="Self_test_in_progress";
+    break;
+  default:msg="Unknown_status ";
+    break;
+  }
+
+  // find the number of columns needed for printing. If in use, the
+  // start/end of span being read-scanned...
+  if (log->currentspan>5) {
+    maxl=current;
+    maxr=currentend;
+  }
+  for (i=0; i<5; i++) {
+    uint64_t start=log->span[i].start;
+    uint64_t end  =log->span[i].end; 
+    // ... plus max start/end of each of the five test spans.
+    if (start>maxl)
+      maxl=start;
+    if (end > maxr)
+      maxr=end;
+  }
+  
+  // we need at least 7 characters wide fields to accomodate the
+  // labels
+  if ((field1=snprintf(tmp,64, "%"PRIu64, maxl))<7)
+    field1=7;
+  if ((field2=snprintf(tmp,64, "%"PRIu64, maxr))<7)
+    field2=7;
+
+  // now print the five test spans
+  pout(" SPAN  %*s  %*s  CURRENT_TEST_STATUS\n", field1, "MIN_LBA", field2, "MAX_LBA");
+
+  for (i=0; i<5; i++) {
+    uint64_t start=log->span[i].start;
+    uint64_t end=log->span[i].end;
+    
+    if ((i+1)==(int)log->currentspan)
+      // this span is currently under test
+      pout("    %d  %*"PRIu64"  %*"PRIu64"  %s [%01d0%% left] (%"PRIu64"-%"PRIu64")\n",
+	   i+1, field1, start, field2, end, msg,
+	   (int)(sv->self_test_exec_status & 0x7), current, currentend);
+    else
+      // this span is not currently under test
+      pout("    %d  %*"PRIu64"  %*"PRIu64"  Not_testing\n",
+	   i+1, field1, start, field2, end);
+  }  
+  
+  // if we are currently read-scanning, print LBAs and the status of
+  // the read scan
+  if (log->currentspan>5)
+    pout("%5d  %*"PRIu64"  %*"PRIu64"  Read_scanning %s\n",
+	 (int)log->currentspan, field1, current, field2, currentend,
+	 OfflineDataCollectionStatus(sv->offline_data_collection_status));
+  
+  /* Print selective self-test flags.  Possible flag combinations are
+     (numbering bits from 0-15):
+     Bit-1 Bit-3   Bit-4
+     Scan  Pending Active
+     0     *       *       Don't scan
+     1     0       0       Will carry out scan after selective test
+     1     1       0       Waiting to carry out scan after powerup
+     1     0       1       Currently scanning       
+     1     1       1       Currently scanning
+  */
+  
+  pout("Selective self-test flags (0x%x):\n", (unsigned int)log->flags);
+  if (log->flags & SELECTIVE_FLAG_DOSCAN) {
+    if (log->flags & SELECTIVE_FLAG_ACTIVE)
+      pout("  Currently read-scanning the remainder of the disk.\n");
+    else if (log->flags & SELECTIVE_FLAG_PENDING)
+      pout("  Read-scan of remainder of disk interrupted; will resume %d min after power-up.\n",
+	   (int)log->pendingtime);
+    else
+      pout("  After scanning selected spans, read-scan remainder of disk.\n");
+  }
+  else
+    pout("  After scanning selected spans, do NOT read-scan remainder of disk.\n");
+  
+  // print pending time
+  pout("If Selective self-test is pending on power-up, resume after %d minute delay.\n",
+       (int)log->pendingtime);
+
+  return; 
+}
+
 // return value is:
 // bottom 8 bits: number of entries found where self-test showed an error
 // remaining bits: if nonzero, power on hours of last self-test where error was found
@@ -1097,7 +1222,7 @@ int ataPrintSmartSelfTestlog(struct ata_smart_selftestlog *data,int allentries){
       if (!errorfound || log->lbafirstfailure==0xffffffff || log->lbafirstfailure==0x00000000)
         sprintf(firstlba,"%s","-");
       else      
-        sprintf(firstlba,"0x%08x",log->lbafirstfailure);
+        sprintf(firstlba,"%u",log->lbafirstfailure);
 
       // print out a header if needed
       if (noheaderprinted && (allentries || errorfound)){
@@ -1540,7 +1665,25 @@ int ataPrintMain (int fd){
       pout("\n");
     }
   }
-  
+
+  // Print SMART selective self-test log
+  if (con->selectivetestlog){
+    struct ata_selective_self_test_log log;
+    
+    if (!isSupportSelectiveSelfTest(&smartval))
+      pout("Device does not support Selective Self Tests/Logging\n");
+    else if(ataReadSelectiveSelfTestLog(fd, &log)) {
+      pout("Smartctl: SMART Selective Self Test Log Read Failed\n");
+      failuretest(OPTIONAL_CMD, returnval|=FAILSMART);
+    }
+    else {
+      PRINT_ON(con);
+      ataPrintSelectiveSelfTestLog(&log, &smartval);
+      PRINT_OFF(con);
+      pout("\n");
+    }
+  }
+
   // START OF THE TESTING SECTION OF THE CODE.  IF NO TESTING, RETURN
   if (con->testcase==-1)
     return returnval;
@@ -1571,15 +1714,13 @@ int ataPrintMain (int fd){
       failuretest(OPTIONAL_CMD, returnval|=FAILSMART);
     }
     break;
-#if DEVELOP_SELECTIVE_SELF_TEST
   case SELECTIVE_SELF_TEST:
   case SELECTIVE_CAPTIVE_SELF_TEST:
     if (!isSupportSelectiveSelfTest(&smartval)){
       pout("Warning: device does not support Selective Self-Test functions.\n\n");
-      failuretest(OPTIONAL_CMD, returnval|=FAILSMART);
+      failuretest(MANDATORY_CMD, returnval|=FAILSMART);
     }
     break;
-#endif
   default:
     pout("Internal error in smartctl: con->testcase==%d not recognized\n", (int)con->testcase);
     pout("Please contact smartmontools developers at %s.\n", PACKAGE_BUGREPORT);
@@ -1588,40 +1729,42 @@ int ataPrintMain (int fd){
 
   // Now do the test.  Note ataSmartTest prints its own error/success
   // messages
-  if (ataSmartTest(fd, con->testcase))
+  if (ataSmartTest(fd, con->testcase, &smartval))
     failuretest(OPTIONAL_CMD, returnval|=FAILSMART);
-  
-  // Tell user how long test will take to complete.  This is tricky
-  // because in the case of an Offline Full Scan, the completion timer
-  // is volatile, and needs to be read AFTER the command is given. If
-  // this will interrupt the Offline Full Scan, we don't do it, just
-  // warn user.
-  if (con->testcase==OFFLINE_FULL_SCAN){
-    if (isSupportOfflineAbort(&smartval))
-      pout("Note: giving further SMART commands will abort Offline testing\n");
-    else if (ataReadSmartValues(fd, &smartval)){
-      pout("Smartctl: SMART Read Values failed.\n");
-      failuretest(OPTIONAL_CMD, returnval|=FAILSMART);
+  else {  
+    // Tell user how long test will take to complete.  This is tricky
+    // because in the case of an Offline Full Scan, the completion
+    // timer is volatile, and needs to be read AFTER the command is
+    // given. If this will interrupt the Offline Full Scan, we don't
+    // do it, just warn user.
+    if (con->testcase==OFFLINE_FULL_SCAN){
+      if (isSupportOfflineAbort(&smartval))
+	pout("Note: giving further SMART commands will abort Offline testing\n");
+      else if (ataReadSmartValues(fd, &smartval)){
+	pout("Smartctl: SMART Read Values failed.\n");
+	failuretest(OPTIONAL_CMD, returnval|=FAILSMART);
+      }
+    }
+    
+    // Now say how long the test will take to complete
+    if ((timewait=TestTime(&smartval,con->testcase))){ 
+      time_t t=time(NULL);
+      if (con->testcase==OFFLINE_FULL_SCAN) {
+	t+=timewait;
+	pout("Please wait %d seconds for test to complete.\n", (int)timewait);
+      } else {
+	t+=timewait*60;
+	pout("Please wait %d minutes for test to complete.\n", (int)timewait);
+      }
+      pout("Test will complete after %s\n", ctime(&t));
+      
+      if (con->testcase!=SHORT_CAPTIVE_SELF_TEST && 
+	  con->testcase!=EXTEND_CAPTIVE_SELF_TEST && 
+	  con->testcase!=CONVEYANCE_CAPTIVE_SELF_TEST && 
+	  con->testcase!=SELECTIVE_CAPTIVE_SELF_TEST)
+	pout("Use smartctl -X to abort test.\n"); 
     }
   }
-  
-  // Now say how long the test will take to complete
-  if ((timewait=TestTime(&smartval,con->testcase))){ 
-    time_t t=time(NULL);
-    if (con->testcase==OFFLINE_FULL_SCAN) {
-      t+=timewait;
-      pout("Please wait %d seconds for test to complete.\n", (int)timewait);
-    } else {
-      t+=timewait*60;
-      pout("Please wait %d minutes for test to complete.\n", (int)timewait);
-    }
-    pout("Test will complete after %s\n", ctime(&t));
-    
-    if (con->testcase!=SHORT_CAPTIVE_SELF_TEST && 
-        con->testcase!=EXTEND_CAPTIVE_SELF_TEST && 
-        con->testcase!=CONVEYANCE_CAPTIVE_SELF_TEST && 
-        con->testcase!=SELECTIVE_CAPTIVE_SELF_TEST)
-      pout("Use smartctl -X to abort test.\n"); 
-  }    
+
   return returnval;
 }
