@@ -1,11 +1,26 @@
-#!/bin/sh -e
-# $Id: autogen.sh,v 1.4 2004/03/04 02:44:39 pervalidus Exp $
+#!/bin/sh
+# $Id: autogen.sh,v 1.8 2004/05/27 15:23:16 card_captor Exp $
 #
 # Generate ./configure from config.in and Makefile.in from Makefile.am.
 # This also adds files like missing,depcomp,install-sh to the source
 # direcory. To update these files at a later date use:
 #	autoreconf -f -i -v
 
+
+# Cygwin?
+test -x /usr/bin/uname && /usr/bin/uname | grep -i CYGWIN >/dev/null &&
+{
+    # Enable strict case checking
+    # (to avoid e.g "DIST_COMMON = ... ChangeLog ..." in Makefile.in)
+    export CYGWIN="${CYGWIN}${CYGWIN:+ }check_case:strict"
+
+    # Check for Unix text file type
+    echo > dostest.tmp
+    test "`wc -c < dostest.tmp`" -eq 1 ||
+        echo "Warning: DOS text file type set, 'make dist' and related targets will not work."
+    rm -f dostest.tmp
+}
+
 typep()
 {
     cmd=$1 ; TMP=$IFS ; IFS=: ; set $PATH
@@ -21,10 +36,10 @@ typep()
     return 1
 }
 
-test -x "$AUTOMAKE" || AUTOMAKE=`typep automake-1.8` || AUTOMAKE=`typep automake-1.7` || AUTOMAKE=`typep automake-1.6` ||
+test -x "$AUTOMAKE" || AUTOMAKE=`typep automake-1.8` || AUTOMAKE=`typep automake-1.7` || AUTOMAKE=`typep automake17` ||
 {
 echo
-echo "You must have at least GNU Automake 1.6 (up to 1.8.x) installed"
+echo "You must have at least GNU Automake 1.7 (up to 1.8.x) installed"
 echo "in order to bootstrap smartmontools from CVS. Download the"
 echo "appropriate package for your distribution, or the source tarball"
 echo "from ftp://ftp.gnu.org/gnu/automake/ ."
@@ -39,13 +54,15 @@ exit 1;
 test -x "$ACLOCAL" || ACLOCAL="aclocal`echo "$AUTOMAKE" | sed 's/.*automake//'`" && ACLOCAL=`typep "$ACLOCAL"` ||
 {
 echo
-echo "autogen.sh found automake-1.6, automake-1.7, or automake-1.8 in"
-echo "your PATH, but not the respective aclocal-1.6, aclocal-1.7, or"
+echo "autogen.sh found automake-1.7, or automake-1.8 in"
+echo "your PATH, but not the respective aclocal-1.7, or"
 echo "aclocal-1.8. Your installation of GNU Automake is broken or"
 echo "incomplete."
 exit 2;
 }
 
+set -e	# stops on error status
+
 ${ACLOCAL}
 autoheader
 ${AUTOMAKE} --add-missing --copy --foreign
