@@ -25,10 +25,7 @@
 #ifndef EXTERN_H_
 #define EXTERN_H_
 
-#define EXTERN_H_CVSID "$Id: extern.h,v 1.34 2004/03/04 13:23:05 ballen4705 Exp $\n"
-
-// For development and testing of Selective self-test code
-#define DEVELOP_SELECTIVE_SELF_TEST 0
+#define EXTERN_H_CVSID "$Id: extern.h,v 1.37 2004/04/17 11:57:43 ballen4705 Exp $\n"
 
 // Possible values for fixfirmwarebug.  If use has NOT specified -F at
 // all, then value is 0.
@@ -41,60 +38,55 @@
 // global variables, this should be the only place that you need to
 // add them.
 typedef struct smartmonctrl_s {
+  // spans for selective self-test
+  uint64_t smartselectivespan[5][2];
+  // number of spans
+  int smartselectivenumspans;
+  int           testcase;
+  // one plus time in minutes to wait after powerup before restarting
+  // interrupted offline scan after selective self-test.
+  int  pendingtime;
+  // run offline scan after selective self-test.  0: don't change, 1:
+  // turn off scan after selective self-test, 2: turn on scan after
+  // selective self-test.
+  unsigned char scanafterselect;
   unsigned char driveinfo;
   unsigned char checksmart;
   unsigned char smartvendorattrib;
   unsigned char generalsmartvalues;
-
   unsigned char smartlogdirectory;
   unsigned char smartselftestlog;
+  unsigned char selectivetestlog;
   unsigned char smarterrorlog;
   unsigned char smartdisable;
-
   unsigned char smartenable; 
   unsigned char smartstatus;
   unsigned char smartexeoffimmediate;
   unsigned char smartshortselftest;
-
   unsigned char smartextendselftest;
   unsigned char smartconveyanceselftest;
-#if DEVELOP_SELECTIVE_SELF_TEST
   unsigned char smartselectiveselftest;
-#endif
   unsigned char smartshortcapselftest;
   unsigned char smartextendcapselftest;
-
   unsigned char smartconveyancecapselftest;
-#if DEVELOP_SELECTIVE_SELF_TEST
   unsigned char smartselectivecapselftest;
-#endif
   unsigned char smartselftestabort;
   unsigned char smartautoofflineenable;
   unsigned char smartautoofflinedisable;
-
   unsigned char smartautosaveenable;
   unsigned char smartautosavedisable;
-#if DEVELOP_SELECTIVE_SELF_TEST
-  unsigned long long smartselectivespan[5][2];
-  int smartselectivenumspans;
-#endif
   unsigned char printing_switchable;
   unsigned char dont_print;
-
-  int           testcase;
-
   unsigned char permissive;
   unsigned char conservative;
   unsigned char checksumfail;
   unsigned char checksumignore;
-
   unsigned char reportataioctl;
   unsigned char reportscsiioctl;
   unsigned char fixfirmwarebug;
   // If nonzero, escalade is 1 plus the disk number behind an escalade
   // controller
   unsigned char escalade;
-
   unsigned char ignorepresets;
   unsigned char showpresets;
   // The i'th entry in this array will modify the printed meaning of
@@ -103,10 +95,6 @@ typedef struct smartmonctrl_s {
   // attributedefs[i] is nonzero, it means that the i'th attribute has
   // a non-default meaning.  See the ataPrintSmartAttribName and
   // and parse_attribute_def functions.
-
-  char notused1;    // make structure pack cleanly without holes
-  char notused2;
-
   unsigned char attributedefs[256];
 } smartmonctrl;
 
