@@ -18,17 +18,16 @@
  */
 
 #include <stdio.h>
-#include <sys/types.h>
-#include <regex.h>
 #include "atacmds.h"
 #include "ataprint.h"
 #include "extern.h"
+#include "int64.h"
 #include "knowndrives.h"
-#include "utility.h"
+#include "utility.h" // includes <regex.h>
 #include "config.h"
 
-const char *knowndrives_c_cvsid="$Id: knowndrives.c,v 1.96 2004/03/06 19:43:17 ballen4705 Exp $"
-                                ATACMDS_H_CVSID ATAPRINT_H_CVSID CONFIG_H_CVSID EXTERN_H_CVSID KNOWNDRIVES_H_CVSID UTILITY_H_CVSID;
+const char *knowndrives_c_cvsid="$Id: knowndrives.c,v 1.111 2004/07/05 07:41:52 ballen4705 Exp $"
+ATACMDS_H_CVSID ATAPRINT_H_CVSID CONFIG_H_CVSID EXTERN_H_CVSID INT64_H_CVSID KNOWNDRIVES_H_CVSID UTILITY_H_CVSID;
 
 #define MODEL_STRING_LENGTH                         40
 #define FIRMWARE_STRING_LENGTH                       8
@@ -51,7 +50,7 @@ const char *knowndrives_c_cvsid="$Id: knowndrives.c,v 1.96 2004/03/06 19:43:17 b
 /* Arrays of preset vendor-specific attribute options for use in
  * knowndrives[]. */
 
-extern long long bytes;
+extern int64_t bytes;
 
 // to hold onto exit code for atexit routine
 extern int exitstatus;
@@ -113,8 +112,10 @@ const unsigned char vendoropts_Hitachi_DK23XX[][2] = {
 const char same_as_minus_F[]="Fixes byte order in some SMART data (same as -F samsung)";
 const char same_as_minus_F2[]="Fixes byte order in some SMART data (same as -F samsung2)";
 
-const char may_need_minus_F_disabled[]="May need -F samsung disabled; see manual for details.";
+const char may_need_minus_F_disabled[] ="May need -F samsung disabled; see manual for details.";
 const char may_need_minus_F2_disabled[]="May need -F samsung2 disabled; see manual for details.";
+const char may_need_minus_F2_enabled[] ="May need -F samsung2 enabled; see manual for details.";
+const char may_need_minus_F_enabled[]  ="May need -F samsung or -F samsung2 enabled; see manual for details.";
 
 /* Special-purpose functions for use in knowndrives[]. */
 void specialpurpose_reverse_samsung(smartmonctrl *con)
@@ -143,21 +144,35 @@ const drivesettings knowndrives[] = {
     "IC35L0[12346]0AVER07",
     ".*",
     "IBM Deskstar 60GXP drives may need upgraded SMART firmware.\n"
-      "Please see http://www.geocities.com/dtla_update/index.html#rel",
+    "Please see http://www.geocities.com/dtla_update/index.html#rel and\n"
+    "http://www-3.ibm.com/pc/support/site.wss/document.do?lndocid=MIGR-42215",
     NULL, NULL, NULL
   },
   { // IBM Deskstar 40GV & 75GXP series (A5AA/A6AA firmware)
-    "(IBM-)?DTLA-30[57]0[123467][05]$",
+    "(IBM-)?DTLA-30[57]0[123467][05]",
     "^T[WX][123468A]OA[56]AA$",
     NULL, NULL, NULL, NULL
   },
   { // IBM Deskstar 40GV & 75GXP series (all other firmware)
-    "(IBM-)?DTLA-30[57]0[123467][05]$",
+    "(IBM-)?DTLA-30[57]0[123467][05]",
     ".*",
     "IBM Deskstar 40GV and 75GXP drives may need upgraded SMART firmware.\n"
-      "Please see http://www.geocities.com/dtla_update/",
+    "Please see http://www.geocities.com/dtla_update/ and\n"
+    "http://www-3.ibm.com/pc/support/site.wss/document.do?lndocid=MIGR-42215",
     NULL, NULL, NULL
   },
+  { // ExcelStor J240
+    "^ExcelStor Technology J240$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
+  { // Fujitsu MPB series
+    "^FUJITSU MPB....ATU?$",
+    ".*",
+    NULL,
+    vendoropts_9_seconds,
+    NULL, NULL
+  },
   { // Fujitsu MPD and MPE series
     "^FUJITSU MP[DE]....A[HTE]$",
     ".*",
@@ -172,6 +187,13 @@ const drivesettings knowndrives[] = {
     vendoropts_9_seconds,
     NULL, NULL
   },
+  { // Fujitsu MPG series
+    "^FUJITSU MPG3(102A(H|T  E)|204A(H|[HT]  E)|307A(H  E|T)|409A[HT]  E)$",
+    ".*",
+    NULL,
+    vendoropts_9_seconds,
+    NULL, NULL
+  },
   { // Fujitsu MPC series
     "^FUJITSU MPC3(032AT|043AT|045AH|064A[HT]|084AT|096AT|102AT)$",
     ".*",
@@ -193,9 +215,9 @@ const drivesettings knowndrives[] = {
     vendoropts_Fujitsu_MHR2040AT,
     NULL, NULL
   },
-  { // Fujitsu MHS2020AT
-    "^FUJITSU MHS2020AT$",
-    ".*",    // Tested on 8004
+  { // Fujitsu MHSxxxxAT family
+    "^FUJITSU MHS20[6432]0AT(  .)?$",
+    ".*",
     NULL,
     vendoropts_Fujitsu_MHS2020AT,
     NULL, NULL
@@ -208,6 +230,13 @@ const drivesettings knowndrives[] = {
     vendoropts_9_seconds,
     NULL, NULL
   },
+  { // Fujitsu MHTxxxxAT family
+    "^FUJITSU MHT20[23468]0AT$",
+    ".*",
+    NULL,
+    vendoropts_9_seconds,
+    NULL, NULL
+  },
   { // Samsung SV4012H (known firmware)
     "^SAMSUNG SV4012H$",
     "^RM100-08$",
@@ -216,30 +245,6 @@ const drivesettings knowndrives[] = {
     specialpurpose_reverse_samsung,
     same_as_minus_F
   },
-  { // SAMSUNG SP1213N (TL100-23 firmware)
-    "^SAMSUNG SP1213N$",
-    "^TL100-23$",
-    NULL,
-    vendoropts_Samsung_SV4012H,
-    specialpurpose_reverse_samsung2,
-    same_as_minus_F2
-  },
-  { // SAMSUNG SP0802N (TK100-23 firmware)
-    "^SAMSUNG SP0802N$",
-    "^TK100-23$",
-    NULL,
-    vendoropts_Samsung_SV4012H,
-    specialpurpose_reverse_samsung2,
-    same_as_minus_F2
-  },
-  { // Any other Samsung disk with *-23 firmware
-    "^SAMSUNG .*$",
-    ".*-23$",
-    may_need_minus_F2_disabled,
-    vendoropts_Samsung_SV4012H,
-    specialpurpose_reverse_samsung2,
-    same_as_minus_F2
-  },
   { // Samsung SV4012H (all other firmware)
     "^SAMSUNG SV4012H$",
     ".*",
@@ -248,22 +253,6 @@ const drivesettings knowndrives[] = {
     specialpurpose_reverse_samsung,
     same_as_minus_F
   },
-  { // Samsung SV1204H (known firmware)
-    "^SAMSUNG SV1204H$",
-    "^RK100-1[3-5]$",
-    NULL,
-    vendoropts_Samsung_SV1204H,
-    specialpurpose_reverse_samsung,
-    same_as_minus_F
-  },
-  { //Samsung SV1204H (all other firmware)
-    "^SAMSUNG SV1204H$",
-    ".*",
-    may_need_minus_F_disabled,
-    vendoropts_Samsung_SV1204H,
-    specialpurpose_reverse_samsung,
-    same_as_minus_F
-  },
   { // Samsung SV0412H (known firmware)
     "^SAMSUNG SV0412H$",
     "^SK100-01$",
@@ -280,14 +269,23 @@ const drivesettings knowndrives[] = {
     specialpurpose_reverse_samsung,
     same_as_minus_F
   },
-  { //Samsung SP1604N, tested with FW TM100-23
-    "^SAMSUNG SP1604N$",
-    ".*",
+  { // Samsung SV1204H (known firmware)
+    "^SAMSUNG SV1204H$",
+    "^RK100-1[3-5]$",
     NULL,
-    vendoropts_Samsung_SV4012H,
-    NULL,NULL
+    vendoropts_Samsung_SV1204H,
+    specialpurpose_reverse_samsung,
+    same_as_minus_F
   },
-  { //SAMSUNG SV0322A with FW JK200-35
+  { //Samsung SV1204H (all other firmware)
+    "^SAMSUNG SV1204H$",
+    ".*",
+    may_need_minus_F_disabled,
+    vendoropts_Samsung_SV1204H,
+    specialpurpose_reverse_samsung,
+    same_as_minus_F
+  },
+  { //SAMSUNG SV0322A tested with FW JK200-35
     "^SAMSUNG SV0322A$",
     ".*",
     NULL,
@@ -295,13 +293,6 @@ const drivesettings knowndrives[] = {
     NULL,
     NULL
   },
-  { // All Samsung drives with '.*-25' firmware
-    "^SAMSUNG.*",
-    ".*-25$",
-    "May need -F samsung2 enabled; see manual for details.\n",
-    vendoropts_9_halfminutes,
-    NULL, NULL
-  },
   { // SAMSUNG SP40A2H with RR100-07 firmware
     "^SAMSUNG SP40A2H$",
     "^RR100-07$",
@@ -310,21 +301,75 @@ const drivesettings knowndrives[] = {
     specialpurpose_reverse_samsung,
     same_as_minus_F
   },
+  { 
+    // Any other Samsung disk with *-23 *-24 firmware
+    // SAMSUNG SP1213N (TL100-23 firmware)
+    // SAMSUNG SP0802N (TK100-23 firmware)
+    // Samsung SP1604N, tested with FW TM100-23 and TM100-24
+    "^SAMSUNG .*$",
+    ".*-2[34]$",
+    NULL,
+    vendoropts_Samsung_SV4012H,
+    specialpurpose_reverse_samsung2,
+    same_as_minus_F2
+  },
+  { // All Samsung drives with '.*-25' firmware
+    "^SAMSUNG.*",
+    ".*-25$",
+    may_need_minus_F2_disabled,
+    vendoropts_Samsung_SV4012H,
+    specialpurpose_reverse_samsung2,
+    same_as_minus_F2
+  },
+  { // All Samsung drives with '.*-26 or later (currently to -39)' firmware
+    "^SAMSUNG.*",
+    ".*-(2[6789]|3[0-9])$",
+    NULL,
+    vendoropts_Samsung_SV4012H,
+    NULL,
+    NULL
+  },
   { // Samsung ALL OTHER DRIVES
     "^SAMSUNG.*",
     ".*",
-    "May need -F samsung or -F samsung2 enabled; see manual for details.\n",
+    may_need_minus_F_enabled,
     NULL, NULL, NULL
   },
+  { // Maxtor Fireball 541DX family
+    "^Maxtor 2B0(0[468]|1[05]|20)H1$",
+    ".*",
+    NULL,
+    vendoropts_Maxtor_4D080H4,
+    NULL, NULL
+  },
+  { // Maxtor DiamondMax 3400 Ultra ATA family
+    "^Maxtor 9(1(360|350|202)D8|1190D7|10[12]0D6|0840D5|06[48]0D4|0510D3|1(350|202)E8|1010E6|0840E5|0640E4)$",
+    ".*",
+    NULL,
+    vendoropts_9_minutes,
+    NULL, NULL
+  },
+  { // Maxtor DiamondMax D540X-4G family
+    "^Maxtor 4G(120J6|160J[68])$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
+  { // Maxtor DiamondMax D540X-4K family
+    "^MAXTOR 4K(020H1|040H2|060H3|080H4)$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
   { // Maxtor DiamondMax Plus D740X family
     "^MAXTOR 6L0(20[JL]1|40[JL]2|60[JL]3|80[JL]4)$",
     ".*",
     NULL, NULL, NULL, NULL
   },
-  { // Maxtor 4K080H4
-    "^MAXTOR 4K080H4$",
+  { // Maxtor DiamondMax Plus 6800 Ultra ATA 66 family
+    "^Maxtor 9(2732U8|2390U7|2049U6|1707U5|1366U4|1024U3|0845U3|0683U2)$",
     ".*",
-    NULL, NULL, NULL, NULL
+    NULL,
+    vendoropts_9_minutes,
+    NULL, NULL
   },
   { // Maxtor 4D080H4
     "^Maxtor (4D080H4|4G120J6)$",
@@ -333,8 +378,8 @@ const drivesettings knowndrives[] = {
     vendoropts_Maxtor_4D080H4,
     NULL, NULL
   },
-  { // Maxtor 4R080J0 and 4R080L0
-    "^Maxtor 4R080[JL]0$",
+  { // Maxtor DiamondMax 16 family
+    "^Maxtor 4(R0[68]0[JL]|R1[26]0L|A160J)0$",
     ".*",
     NULL,
     vendoropts_9_minutes,
@@ -368,6 +413,13 @@ const drivesettings knowndrives[] = {
     vendoropts_9_minutes,
     NULL, NULL
   },
+  { // Maxtor DiamondMax 40 ATA 66 series
+    "^Maxtor 9(0684U2|1024U2|1362U3|1536U3|2049U4|2562U5|3073U6|4098U8)$",
+    ".*",
+    NULL,
+    vendoropts_9_minutes,
+    NULL, NULL
+  },
   { // Maxtor DiamondMax Plus 40 series (Ultra ATA 66 and Ultra ATA 100)
     "^Maxtor (54098[UH]8|53073[UH]6|52732[UH]6|52049[UH]4|51536[UH]3|51369[UH]3|51024[UH]2)$",
     ".*",
@@ -375,6 +427,20 @@ const drivesettings knowndrives[] = {
     vendoropts_9_minutes,
     NULL, NULL
   },
+  { // Maxtor DiamondMax 40 VL Ultra ATA 100 series
+    "^Maxtor 3(1024H1|1535H2|2049H2|3073H3|4098H4)( B)?$",
+    ".*",
+    NULL,
+    vendoropts_9_minutes,
+    NULL, NULL
+  },
+  { // Maxtor DiamondMax Plus 45 Ulta ATA 100 family
+    "^Maxtor 5(4610H6|4098H6|3073H4|2049H3|1536H2|1369H2|1023H2)$",
+    ".*",
+    NULL,
+    vendoropts_9_minutes,
+    NULL, NULL
+  },
   { // Maxtor DiamondMax Plus 60 family
     "^Maxtor 5T0(60H6|40H4|30H3|20H2|10H1)$",
     ".*",
@@ -424,8 +490,15 @@ const drivesettings knowndrives[] = {
     vendoropts_9_minutes,
     NULL, NULL
   },
-  { // HITACHI Travelstar DK23XX series
-    "^HITACHI_DK23..-..$",
+  { // HITACHI_DK14FA-20B
+    "^HITACHI_DK14FA-20B$",
+    ".*",
+    NULL,
+    vendoropts_Hitachi_DK23XX,
+    NULL, NULL
+  },
+  { // HITACHI Travelstar DK23XX/DK23XXB series
+    "^HITACHI_DK23..-..B?$",
     ".*",
     NULL,
     vendoropts_Hitachi_DK23XX,
@@ -436,6 +509,11 @@ const drivesettings knowndrives[] = {
     ".*",
     NULL, NULL, NULL, NULL 
   },
+  { // IBM Deskstar 25GP and 22GXP family
+    "^IBM-DJNA-3(5(101|152|203|250)|7(091|135|180|220))0$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
   { // IBM Travelstar 25GS, 18GT, and 12GN family
     "^IBM-DARA-2(25|18|15|12|09|06)000$",
     ".*",
@@ -451,6 +529,16 @@ const drivesettings knowndrives[] = {
     ".*",
     NULL, NULL, NULL, NULL 
   },
+  { // IBM/Hitachi Travelstar 40GNX family
+    "^IC25N0[42]0ATC[SX]05-.$",
+    ".*",
+    NULL, NULL, NULL, NULL 
+  },
+  { // Hitachi Travelstar 80GN family
+    "^IC25N0[23468]0ATMR04-.$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
   { // IBM/Hitachi Deskstar 120GXP family
     "^IC35L((020|040|060|080|120)AVVA|0[24]0AVVN)07-[01]$",
     ".*",
@@ -466,6 +554,11 @@ const drivesettings knowndrives[] = {
     ".*",
     NULL, NULL, NULL, NULL
   },
+  { // IBM Travelstar 4LP
+    "^IBM-DTNA-2(180|216)0$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
   { // Hitachi Deskstar 7K250 series
     "^HDS7225((40|80|12|16)VLAT20|(12|16|25)VLAT80|(80|12|16|25)VLSA80)$",
     ".*",
@@ -501,13 +594,33 @@ const drivesettings knowndrives[] = {
     ".*",
     NULL, NULL, NULL, NULL
   },
-  { // Seagate Barracuda U Series family
-    "^ST3(8002|6002|4081|2041)0A$",
+  { // TOSHIBA MK4018GAS
+    "^TOSHIBA MK4018GAS$",
     ".*",
     NULL, NULL, NULL, NULL
   },
-  { // Seagate U Series 5 20413
-    "^ST320413A$",
+  { // Seagate Medalist 8641 family
+    "^ST3(2110|3221|4312|6531|8641)A$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
+  { // Seagate U Series X family
+    "^ST3(10014A(CE)?|20014A)$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
+  { // Seagate U Series 6 family
+    "^ST3(8002|6002|4081|3061|2041)0A$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
+  { // Seagate U Series 5 family
+    "^ST3(40823|30621|20413|15311|10211)A$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
+  { // Seagate U8 family
+    "^ST3(8410|4313|17221|13021)A$",
     ".*",
     NULL, NULL, NULL, NULL
   },
@@ -536,18 +649,27 @@ const drivesettings knowndrives[] = {
     ".*",
     NULL, NULL, NULL, NULL
   },
-  { // Western Digital Protege WD400EB
+  { // Seagate Medalist 17242, 13032, 10232, 8422, and 4312
+    "^ST3(1724|1303|1023|842|431)2A$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
+  { // Western Digital Protege
   /* Western Digital drives with this comment all appear to use Attribute 9 in
    * a  non-standard manner.  These entries may need to be updated when it
-   * is understood exactly how Attribute 9 should be interpreted. */
-    "^WDC WD400EB-.*$",
+   * is understood exactly how Attribute 9 should be interpreted.
+   * UPDATE: this is probably explained by the WD firmware bug described in the
+   * smartmontools FAQ */
+    "^WDC WD([2468]00E|1[26]00A)B-.*$",
     ".*",
     NULL, NULL, NULL, NULL
   },
   { // Western Digital Caviar family
   /* Western Digital drives with this comment all appear to use Attribute 9 in
    * a  non-standard manner.  These entries may need to be updated when it
-   * is understood exactly how Attribute 9 should be interpreted. */
+   * is understood exactly how Attribute 9 should be interpreted.
+   * UPDATE: this is probably explained by the WD firmware bug described in the
+   * smartmontools FAQ */
     "^WDC WD(2|3|4|6|8|10|12|16|18|20|25)00BB-.*$",
     ".*",
     NULL, NULL, NULL, NULL
@@ -555,7 +677,9 @@ const drivesettings knowndrives[] = {
   { // Western Digital Caviar WDxxxAB series
   /* Western Digital drives with this comment all appear to use Attribute 9 in
    * a  non-standard manner.  These entries may need to be updated when it
-   * is understood exactly how Attribute 9 should be interpreted. */
+   * is understood exactly how Attribute 9 should be interpreted.
+   * UPDATE: this is probably explained by the WD firmware bug described in the
+   * smartmontools FAQ */
     "^WDC WD(3|4|6)00AB-.*$",
     ".*",
     NULL, NULL, NULL, NULL
@@ -563,15 +687,19 @@ const drivesettings knowndrives[] = {
   { // Western Digital Caviar WDxxxAA series
   /* Western Digital drives with this comment all appear to use Attribute 9 in
    * a  non-standard manner.  These entries may need to be updated when it
-   * is understood exactly how Attribute 9 should be interpreted. */
-    "^WDC WD(64|84|102|136|205|307)AA$",
+   * is understood exactly how Attribute 9 should be interpreted.
+   * UPDATE: this is probably explained by the WD firmware bug described in the
+   * smartmontools FAQ */
+    "^WDC WD(64|84|102|136|205|307)AA(-.*)?$",
     ".*",
     NULL, NULL, NULL, NULL
   },
   { // Western Digital Caviar WDxxxBA series
   /* Western Digital drives with this comment all appear to use Attribute 9 in
    * a  non-standard manner.  These entries may need to be updated when it
-   * is understood exactly how Attribute 9 should be interpreted. */
+   * is understood exactly how Attribute 9 should be interpreted.
+   * UPDATE: this is probably explained by the WD firmware bug described in the
+   * smartmontools FAQ */
     "^WDC WD(102|136|153|205)BA$",
     ".*",
     NULL, NULL, NULL, NULL
@@ -584,7 +712,9 @@ const drivesettings knowndrives[] = {
   { // Western Digital Caviar SE family
   /* Western Digital drives with this comment all appear to use Attribute 9 in
    * a  non-standard manner.  These entries may need to be updated when it
-   * is understood exactly how Attribute 9 should be interpreted. */
+   * is understood exactly how Attribute 9 should be interpreted.
+   * UPDATE: this is probably explained by the WD firmware bug described in the
+   * smartmontools FAQ */
     "^WDC WD(4|6|8|10|12|16|18|20|25)00JB-.*$",
     ".*",
     NULL, NULL, NULL, NULL
@@ -609,8 +739,8 @@ const drivesettings knowndrives[] = {
     ".*",
     NULL, NULL, NULL, NULL
   },
-  { // QUANTUM FIREBALLlct20 20 and QUANTUM FIREBALLlct20 40
-    "^QUANTUM FIREBALLlct20 [24]0$",
+  { // QUANTUM FIREBALLlct20 series
+    "^QUANTUM FIREBALLlct20 [234]0$",
     ".*",
     NULL, NULL, NULL, NULL
   },
@@ -619,6 +749,26 @@ const drivesettings knowndrives[] = {
     ".*",
     NULL, NULL, NULL, NULL
   },
+  { // QUANTUM FIREBALLP LM15 and LM30
+    "^QUANTUM FIREBALLP LM(15|30)$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
+  { // QUANTUM FIREBALL CR4.3A
+    "^QUANTUM FIREBALL CR4.3A$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
+  { // QUANTUM FIREBALLP AS10.2 and AS40.0
+    "^QUANTUM FIREBALLP AS(10.2|40.0)$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
+  { // QUANTUM FIREBALL EX6.4A
+    "^QUANTUM FIREBALL EX6.4A$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
   /*------------------------------------------------------------
    *  End of table.  Do not add entries below this marker.
    *------------------------------------------------------------ */
