@@ -14,32 +14,33 @@
  * (for example COPYING); if not, write to the Free
  * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * This code was originally developed as a Senior Thesis by Michael Cornwell
- * at the Concurrent Systems Laboratory (now part of the Storage Systems
- * Research Center), Jack Baskin School of Engineering, University of
- * California, Santa Cruz. http://ssrc.soe.ucsc.edu/
- *
  */
 
 #include "config.h"
 #include "atacmds.h"
-#include "scsicmds.h"
-#include "utility.h"
-extern long long bytes; // malloc() byte count
-
 #include "extern.h"
 extern smartmonctrl * con; // con->permissive
+#include "int64.h"
+#include "scsicmds.h"
+#include "utility.h"
+extern int64_t bytes; // malloc() byte count
 
 #include <errno.h>
+#ifdef _DEBUG
 #include <assert.h>
+#else
+#define assert(x) /**/
+#endif
 #define WIN32_LEAN_AND_MEAN
 #include <windows.h>
 #include <stddef.h> // offsetof()
+#include <io.h> // access()
 
+#define ARGUSED(x) ((void)(x))
 
 // Needed by '-V' option (CVS versioning) of smartd/smartctl
-const char *os_XXXX_c_cvsid="$Id: os_win32.c,v 1.4 2004/03/04 21:50:14 chrfranke Exp $" \
-ATACMDS_H_CVSID SCSICMDS_H_CVSID UTILITY_H_CVSID EXTERN_H_CVSID;
+const char *os_XXXX_c_cvsid="$Id: os_win32.c,v 1.14 2004/06/24 12:28:39 chrfranke Exp $"
+ATACMDS_H_CVSID CONFIG_H_CVSID EXTERN_H_CVSID INT64_H_CVSID SCSICMDS_H_CVSID UTILITY_H_CVSID;
 
 
 static int ata_open(int drive);
@@ -48,6 +49,7 @@ static unsigned ata_scan(void);
 
 static int aspi_open(unsigned adapter, unsigned id);
 static void aspi_close(int fd);
+static unsigned long aspi_scan(void);
 
 
 static int is_permissive()
@@ -85,40 +87,67 @@ int guess_device_type (const char * dev_name)
 // others each contain null-terminated character strings.
 int make_device_names (char*** devlist, const char* type)
 {
+	unsigned long drives;
+	int i, j, n, sz, scsi;
+	const char * path;
+
 	if (!strcmp(type, "ATA")) {
-		unsigned drives = ata_scan();
-		int i, n, sz;
-		if (!drives)
-			return 0;
-		n = 0;
-		for (i = 0; i <= 9; i++) {
-			if (drives & (1 << i))
-				n++;
-		}
-		assert(n > 0);
-		if (n == 0)
-			return 0;
-		sz = (n+1) * sizeof(char **);
-		*devlist = (char **)malloc(sz); bytes += sz;
-		for (i = 0; i < n; i++) {
-			sz = sizeof("/dev/hda");
-			char * s = (char *)malloc(sz); bytes += sz;
-			strcpy(s, "/dev/hda"); s[sz-2] += i;
-			(*devlist)[i] = s;
-		}
-		(*devlist)[n] = 0;
-		return n;
+		// bit i set => drive i present
+		drives = ata_scan();
+		path = "/dev/hda";
+		scsi = 0;
 	}
-
-	if (!strcmp(type, "SCSI")) {
-		return 0; // TODO!
+	else if (!strcmp(type, "SCSI")) {
+		// bit i set => drive with ID (i & 0x7) on adapter (i >> 3) present
+		drives = aspi_scan();
+		path = "/dev/scsi00";
+		scsi = 1;
 	}
+	else
+		return -1;
 
-	return -1;
+	if (!drives)
+		return 0;
+
+	// Count #drives
+	n = 0;
+	for (i = 0; i < 32; i++) {
+		if (drives & (1 << i))
+			n++;
+	}
+	assert(n > 0);
+	if (n == 0)
+		return 0;
+
+	// Alloc devlist
+	assert(scsi || n <= 9);
+	sz = n * sizeof(char **);
+	*devlist = (char **)malloc(sz); bytes += sz;
+
+	// Add devices
+	for (i = j = 0; i < n; i++) {
+		char * s;
+		sz = strlen(path)+1;
+		s = (char *)malloc(sz); bytes += sz;
+		strcpy(s, path);
+		while (j < 32 && !(drives & (1 << j)))
+			j++;
+		assert(j < 32);
+		if (!scsi) {
+			assert(j <= 9);
+			s[sz-2] += j; // /dev/hd[a-j]
+		}
+		else {
+			s[sz-3] += (j >> 3);  // /dev/scsi[0-3].
+			s[sz-2] += (j & 0x7); //          .....[0-7]
+		}
+		(*devlist)[i] = s;
+		j++;
+	}
+	return n;
 }
 
 
-
 // Like open().  Return positive integer handle, only used by
 // functions below.  type="ATA" or "SCSI".  If you need to store extra
 // information about your devices, create a private internal array
@@ -429,7 +458,7 @@ static int ide_pass_through_ioctl(HANDLE hdevice, IDEREGS * regs, char * data, u
 	}
 
 #ifdef _DEBUG
-	pout("DeviceIoControl returns %lu bytes\n", num_out);
+	pout("DeviceIoControl returns %lu (%lu) bytes\n", num_out, buf->DataBufferSize);
 	print_ide_regs(&buf->IdeReg, 1);
 #endif
 
@@ -443,6 +472,40 @@ static int ide_pass_through_ioctl(HANDLE hdevice, IDEREGS * regs, char * data, u
 /////////////////////////////////////////////////////////////////////////////
 
 static HANDLE h_ata_ioctl = 0;
+static int ide_pass_through_broken = 0;
+
+
+// Print SMARTVSD error message, return errno
+
+static int smartvsd_error()
+{
+	char path[MAX_PATH];
+	unsigned len;
+	if (!(5 <= (len = GetSystemDirectoryA(path, MAX_PATH)) && len < MAX_PATH/2))
+		return ENOENT;
+	strcpy(path+len, "\\IOSUBSYS\\SMARTVSD.VXD");
+	if (!access(path, 0)) {
+#ifdef _DEBUG
+		pout("Driver \"%s\" not loaded,\n"
+		     " possibly no IDE/ATA devices present.\n", path);
+#endif
+		return ENOENT;
+	}
+	// Some Windows versions install SMARTVSD.VXD in SYSTEM directory
+	// http://support.microsoft.com/default.aspx?scid=kb;en-us;265854
+	strcpy(path+len, "\\SMARTVSD.VXD");
+	if (!access(path, 0)) {
+		path[len] = 0;
+		pout("SMART driver is not properly installed,\n"
+		     " move SMARTVSD.VXD from \"%s\" to \"%s\\IOSUBSYS\"\n"
+		     " and reboot Windows.\n", path, path);
+		return ENOSYS;
+	}
+	path[len] = 0;
+	pout("SMARTVSD.VXD is missing in folder \"%s\\IOSUBSYS\".\n", path);
+	return ENOENT;
+}
+
 
 static int ata_open(int drive)
 {
@@ -473,14 +536,16 @@ static int ata_open(int drive)
 		snprintf(devpath, sizeof(devpath)-1, "\\\\.\\PhysicalDrive%d", drive);
 
 	// Open device
-	if ((h_ata_ioctl = CreateFile(devpath,
+	if ((h_ata_ioctl = CreateFileA(devpath,
 		GENERIC_READ|GENERIC_WRITE, FILE_SHARE_READ|FILE_SHARE_WRITE,
 		NULL, OPEN_EXISTING, 0, 0)) == INVALID_HANDLE_VALUE) {
-		pout("Cannot open device %s, Error=%ld\n", devpath, GetLastError());
-		if (win9x)
-			pout("Possibly missing SMARTVSD.VXD in Windows directory SYSTEM\\IOSUBSYS.\n");
+		long err = GetLastError();	
+		pout("Cannot open device %s, Error=%ld\n", devpath, err);
+		if (win9x && err == ERROR_FILE_NOT_FOUND)
+			errno = smartvsd_error();
+		else
+			errno = (err == ERROR_FILE_NOT_FOUND ? ENOENT : EPERM);
 		h_ata_ioctl = 0;
-		errno = ENOENT;
 		return -1;
 	}
 
@@ -532,6 +597,7 @@ static int ata_open(int drive)
 
 static void ata_close(int fd)
 {
+	ARGUSED(fd);
 	CloseHandle(h_ata_ioctl);
 	h_ata_ioctl = 0;
 }
@@ -556,7 +622,7 @@ static unsigned ata_scan()
 			snprintf(devpath, sizeof(devpath)-1, "\\\\.\\PhysicalDrive%d", i);
 
 		// Open device
-		if ((h = CreateFile(devpath,
+		if ((h = CreateFileA(devpath,
 			GENERIC_READ|GENERIC_WRITE, FILE_SHARE_READ|FILE_SHARE_WRITE,
 			NULL, OPEN_EXISTING, 0, 0)) == INVALID_HANDLE_VALUE) {
 			if (win9x)
@@ -609,6 +675,11 @@ int ata_command_interface(int fd, smart_command_set command, int select, char *
 	copydata = 0;
 
 	switch (command) {
+	  case CHECK_POWER_MODE:
+	  case WRITE_LOG:
+		// TODO. Not supported by SMART IOCTL
+		errno = ENOSYS;
+		return -1;
 	  case READ_VALUES:
 		regs.bFeaturesReg = ATA_SMART_READ_VALUES;
 		regs.bSectorNumberReg = regs.bSectorCountReg = 1;
@@ -671,9 +742,19 @@ int ata_command_interface(int fd, smart_command_set command, int select, char *
 	if (smart_ioctl(h_ata_ioctl, fd, &regs, data, (copydata?512:0))) {
 		// Read log only supported on Win9x, retry with pass through command
 		// CAUTION: smart_ioctl() MUST NOT change "regs" Parameter in this case
-		if (errno == ENOSYS && command == READ_LOG) {
+		if (errno == ENOSYS && command == READ_LOG && !ide_pass_through_broken) {
 			errno = 0;
-			return ide_pass_through_ioctl(h_ata_ioctl, &regs, data, 512);
+			memset(data, 0, 512);
+			if (ide_pass_through_ioctl(h_ata_ioctl, &regs, data, 512))
+				return -1;
+			if (!nonempty(data, 512)) {
+				// Nothing useful returned => ioctl probably broken
+				pout("IOCTL_IDE_PASS_THROUGH does not work on your version of Windows\n");
+				ide_pass_through_broken = 1; // Do not retry (smartd)
+				errno = ENOSYS;
+				return -1;
+			}
+			return 0;
 		}
 		return -1;
 	}
@@ -703,6 +784,7 @@ int ata_command_interface(int fd, smart_command_set command, int select, char *
 int escalade_command_interface(int fd, int disknum, smart_command_set command, int select, char *data)
 {
 	static int warned = 0;
+	ARGUSED(fd); ARGUSED(disknum); ARGUSED(command); ARGUSED(select); ARGUSED(data);
 	if (!warned) {
 		  pout(
 		"#######################################################################\n"
@@ -748,6 +830,16 @@ typedef struct {
 	unsigned char parameters[16];  // 42: Host adapter unique parmameters
 } ASPI_SRB_INQUIRY;
 
+// SRB for get device type
+
+typedef struct {
+	ASPI_SRB_HEAD h;               // 00: Header
+	unsigned char target_id;       // 08: Target ID
+	unsigned char lun;             // 09: LUN
+	unsigned char devtype;         // 10: Device type
+	unsigned char reserved;        // 11: Reserved
+} ASPI_SRB_DEVTYPE;
+
 // SRB for SCSI I/O
 
 typedef struct {
@@ -774,6 +866,7 @@ typedef struct {
 typedef union {
 	ASPI_SRB_HEAD h;       // Common header
 	ASPI_SRB_INQUIRY q;    // Inquiry
+	ASPI_SRB_DEVTYPE t;    // Device type
 	ASPI_SRB_IO i;         // I/O
 } ASPI_SRB;
 
@@ -800,6 +893,7 @@ typedef union {
 #define ASPI_STATUS_INVALID_COMMAND     0x80
 #define ASPI_STATUS_INVALID_ADAPTER     0x81
 #define ASPI_STATUS_INVALID_TARGET      0x82
+#define ASPI_STATUS_NO_ADAPTERS         0xE8
 
 // Adapter (host) status
 #define ASPI_HSTATUS_NO_ERROR           0x00
@@ -820,12 +914,69 @@ static HINSTANCE h_aspi_dll; // DLL handle
 static UINT (* aspi_entry)(ASPI_SRB * srb); // ASPI entrypoint
 static unsigned num_aspi_adapters;
 
+#ifdef __CYGWIN__
+// h_aspi_dll+aspi_entry is not inherited by Cygwin's fork()
+static DWORD aspi_dll_pid; // PID of DLL owner to detect fork()
+#define aspi_entry_valid() (aspi_entry && (aspi_dll_pid == GetCurrentProcessId()))
+#else
+#define aspi_entry_valid() (!!aspi_entry)
+#endif
 
-static int aspi_open_dll()
+
+static int aspi_call(ASPI_SRB * srb)
 {
-	ASPI_SRB srb;
+	int i;
+	aspi_entry(srb);
+	i = 0;
+	while (((volatile ASPI_SRB *)srb)->h.status == ASPI_STATUS_IN_PROGRESS) {
+		if (++i > 100/*10sek*/) {
+			pout("ASPI Adapter %u: Timeout\n", srb->h.adapter);
+			aspi_entry = 0;
+			h_aspi_dll = INVALID_HANDLE_VALUE;
+			errno = EIO;
+			return -1;
+		}
+#ifdef _DEBUG
+		pout("ASPI Wait %d\n", i);
+#endif
+		Sleep(100);
+	}
+	return 0;
+}
+
+
+// Get ASPI entrypoint from wnaspi32.dll
+
+static FARPROC aspi_get_address(const char * name, int verbose)
+{
+	FARPROC addr;
+	assert(h_aspi_dll && h_aspi_dll != INVALID_HANDLE_VALUE);
+
+	if (!(addr = GetProcAddress(h_aspi_dll, name))) {
+		if (verbose)
+			pout("Missing %s() in WNASPI32.DLL\n", name);
+		aspi_entry = 0;
+		FreeLibrary(h_aspi_dll);
+		h_aspi_dll = INVALID_HANDLE_VALUE;
+		errno = ENOSYS;
+		return 0;
+	}
+	return addr;
+}
+
+
+static int aspi_open_dll(int verbose)
+{
+	UINT (*aspi_info)(void);
+	UINT info, rc;
+
+	assert(!aspi_entry_valid());
 
 	// Check structure layout
+	assert(sizeof(ASPI_SRB_HEAD) == 8);
+	assert(sizeof(ASPI_SRB_INQUIRY) == 58);
+	assert(sizeof(ASPI_SRB_DEVTYPE) == 12);
+	assert(sizeof(ASPI_SRB_IO) == 64+ASPI_SENSE_SIZE);
 	assert(offsetof(ASPI_SRB,h.cmd) == 0);
 	assert(offsetof(ASPI_SRB,h.flags) == 3);
 	assert(offsetof(ASPI_SRB_IO,lun) == 9);
@@ -839,41 +990,48 @@ static int aspi_open_dll()
 		return -1;
 	}
 
-	// Get ASPI entrypoint from winaspi.dll
-	if (!h_aspi_dll && !(h_aspi_dll = LoadLibrary("WNASPI32.DLL"))) {
-		pout("Cannot Load WNASPI32.DLL, Error=%ld\n", GetLastError());
+	// Load ASPI DLL
+	if (!(h_aspi_dll = LoadLibraryA("WNASPI32.DLL"))) {
+		if (verbose)
+			pout("Cannot load WNASPI32.DLL, Error=%ld\n", GetLastError());
 		h_aspi_dll = INVALID_HANDLE_VALUE;
 		errno = ENOENT;
 		return -1;
 	}
 
-	if (!((FARPROC)aspi_entry = GetProcAddress(h_aspi_dll, "SendASPI32Command"))) {
-		pout("Missing SendASPI32Command() in WNASPI32.DLL\n");
-		FreeLibrary(h_aspi_dll); h_aspi_dll = INVALID_HANDLE_VALUE;
+	// Get ASPI entrypoints
+	if (!(aspi_info = (UINT (*)(void))aspi_get_address("GetASPI32SupportInfo", verbose)))
+		return -1;
+	if (!(aspi_entry = (UINT (*)(ASPI_SRB *))aspi_get_address("SendASPI32Command", verbose)))
+		return -1;
+
+	// Init ASPI manager and get number of adapters
+	info = (aspi_info)();
+#ifdef _DEBUG
+	pout("GetASPI32SupportInfo() returns 0x%04x\n", info);
+#endif
+	rc = (info >> 8) & 0xff;
+	if (rc == ASPI_STATUS_NO_ADAPTERS) {
+		num_aspi_adapters = 0;
+	}
+	else if (rc == ASPI_STATUS_NO_ERROR) {
+		num_aspi_adapters = info & 0xff;
+	}
+	else {
+		if (verbose)
+			pout("Got strange 0x%04x from GetASPI32SupportInfo()\n", info);
+		aspi_entry = 0;
+		FreeLibrary(h_aspi_dll);
+		h_aspi_dll = INVALID_HANDLE_VALUE;
 		errno = ENOENT;
 		return -1;
 	}
 
-	// Get number of adapters
-	memset(&srb, 0, sizeof(srb));
-	srb.h.cmd = ASPI_CMD_ADAPTER_INQUIRE;
-	aspi_entry(&srb);
-	while (((volatile ASPI_SRB *)&srb)->h.status == ASPI_STATUS_IN_PROGRESS) {
-		// TODO: Timeout!
-		Sleep(1);
-	}
-	if (srb.h.status != ASPI_STATUS_NO_ERROR) {
-		pout("ASPI Adapter Inquriy failed, Error=0x%02x\n", srb.h.status);
-		if (!is_permissive()) {
-			aspi_entry = 0;
-			FreeLibrary(h_aspi_dll); h_aspi_dll = INVALID_HANDLE_VALUE;
-			errno = ENOSYS;
-			return -1;
-		}
-		srb.q.adapters = 10;
-	}
-
-	num_aspi_adapters = srb.q.adapters;
+#ifdef __CYGWIN__
+	// save PID to detect fork() in aspi_entry_valid()
+	aspi_dll_pid = GetCurrentProcessId();
+#endif
+	assert(aspi_entry_valid());
 	return 0;
 }
 
@@ -882,7 +1040,7 @@ static int aspi_io_call(ASPI_SRB * srb)
 {
 	HANDLE event;
 	// Create event
-	if (!(event = CreateEvent(NULL, FALSE, FALSE, NULL))) {
+	if (!(event = CreateEventA(NULL, FALSE, FALSE, NULL))) {
 		pout("CreateEvent(): Error=%ld\n", GetLastError()); return -EIO;
 	}
 	srb->i.event_handle = event;
@@ -918,8 +1076,8 @@ static int aspi_open(unsigned adapter, unsigned id)
 		return -1;
 	}
 
-	if (!aspi_entry) {
-		if (aspi_open_dll())
+	if (!aspi_entry_valid()) {
+		if (aspi_open_dll(1/*verbose*/))
 			return -1;
 	}
 
@@ -938,6 +1096,60 @@ static int aspi_open(unsigned adapter, unsigned id)
 
 static void aspi_close(int fd)
 {
+	// No FreeLibrary(h_aspi_dll) to prevent problems with ASPI threads
+	ARGUSED(fd);
+}
+
+
+// Scan for SCSI drives, return bitmask [adapter:0-3][id:0-7] of drives present
+
+static unsigned long aspi_scan()
+{
+	unsigned long drives = 0;
+	unsigned ad, nad;
+
+	if (!aspi_entry_valid()) {
+		if (aspi_open_dll(0/*quiet*/))
+			return 0;
+	}
+
+	nad = num_aspi_adapters;
+	if (nad >= 4)
+		nad = 4;
+	for (ad = 0; ad < nad; ad++) {
+		ASPI_SRB srb; int id;
+		// Get adapter name
+		memset(&srb, 0, sizeof(srb));
+		srb.h.cmd = ASPI_CMD_ADAPTER_INQUIRE;
+		srb.h.adapter = ad;
+		if (aspi_call(&srb))
+			return 0;
+#ifdef _DEBUG
+		pout("ASPI Adapter %u: %02x,\"%.16s\"\n", ad, srb.h.status, srb.q.adapter_id);
+#endif
+		if (srb.h.status != ASPI_STATUS_NO_ERROR)
+			continue;
+
+		// Skip ATA/ATAPI devices
+		srb.q.adapter_id[sizeof(srb.q.adapter_id)-1] = 0;
+		if (strstr(srb.q.adapter_id, "ATAPI"))
+			continue;
+
+		for (id = 0; id <= 7; id++) {
+			// Get device type
+			memset(&srb, 0, sizeof(srb));
+			srb.h.cmd = ASPI_CMD_GET_DEVICE_TYPE;
+			srb.h.adapter = ad; srb.i.target_id = id;
+			if (aspi_call(&srb))
+				return 0;
+#ifdef _DEBUG
+			pout("Device type for scsi%u%x: %02x,%02x\n", ad, id, srb.h.status, srb.t.devtype);
+#endif
+			if (srb.h.status == ASPI_STATUS_NO_ERROR && srb.t.devtype == 0x00/*HDD*/)
+				drives |= 1 << ((ad<<3)+id);
+		}
+	}
+	return drives;
 }
 
 
@@ -948,7 +1160,7 @@ int do_scsi_cmnd_io(int fd, struct scsi_cmnd_io * iop, int report)
 {
 	ASPI_SRB srb;
 
-	if (!aspi_entry)
+	if (!aspi_entry_valid())
 		return -EBADF;
 	if (!((fd & ~0xff) == 0x100))
 		return -EBADF;
@@ -1024,7 +1236,7 @@ int do_scsi_cmnd_io(int fd, struct scsi_cmnd_io * iop, int report)
 	if (srb.h.status != ASPI_STATUS_NO_ERROR) {
 		if (   srb.h.status        == ASPI_STATUS_ERROR
 		    && srb.i.host_status   == ASPI_HSTATUS_NO_ERROR
-			&& srb.i.target_status == ASPI_TSTATUS_CHECK_CONDITION) {
+		    && srb.i.target_status == ASPI_TSTATUS_CHECK_CONDITION) {
 			// Sense valid
 			const unsigned char * sense = ASPI_SRB_SENSE(&srb.i, iop->cmnd_len);
 			int len = (ASPI_SENSE_SIZE < iop->max_sense_len ? ASPI_SENSE_SIZE : iop->max_sense_len);
