@@ -40,13 +40,15 @@
 
 #include <stdio.h>
 #include <string.h>
-#include <unistd.h>
-#include <sys/ioctl.h>
+
+#include "config.h"
+#include "int64.h"
+#include "extern.h"
 #include "scsicmds.h"
 #include "utility.h"
-#include "extern.h"
 
-const char *scsicmds_c_cvsid="$Id: scsicmds.c,v 1.70 2004/01/02 16:05:25 ballen4705 Exp $" EXTERN_H_CVSID SCSICMDS_H_CVSID;
+const char *scsicmds_c_cvsid="$Id: scsicmds.c,v 1.75 2004/04/30 06:15:49 dpgilbert Exp $"
+CONFIG_H_CVSID EXTERN_H_CVSID INT64_H_CVSID SCSICMDS_H_CVSID UTILITY_H_CVSID;
 
 /* for passing global control variables */
 extern smartmonctrl *con;
@@ -249,6 +251,7 @@ int scsiLogSense(int device, int pagenum, UINT8 *pBuf, int bufLen,
         io_hdr.cmnd_len = sizeof(cdb);
         io_hdr.sensep = sense;
         io_hdr.max_sense_len = sizeof(sense);
+        io_hdr.timeout = SCSI_TIMEOUT_DEFAULT;
     
         status = do_scsi_cmnd_io(device, &io_hdr, con->reportscsiioctl);
         if (0 != status)
@@ -282,6 +285,7 @@ int scsiLogSense(int device, int pagenum, UINT8 *pBuf, int bufLen,
     io_hdr.cmnd_len = sizeof(cdb);
     io_hdr.sensep = sense;
     io_hdr.max_sense_len = sizeof(sense);
+    io_hdr.timeout = SCSI_TIMEOUT_DEFAULT;
 
     status = do_scsi_cmnd_io(device, &io_hdr, con->reportscsiioctl);
     if (0 != status)
@@ -301,7 +305,7 @@ int scsiLogSense(int device, int pagenum, UINT8 *pBuf, int bufLen,
 /* Send MODE SENSE (6 byte) command. Returns 0 if ok, 1 if NOT READY,
  * 2 if command not supported (then MODE SENSE(10) should be supported),
  * 3 if field in command not supported or returns negated errno. 
- * SPC sections 7.9 and 8.4 */
+ * SPC sections 7.9 and 8.4 [mode subpage==0] */
 int scsiModeSense(int device, int pagenum, int pc, UINT8 *pBuf, int bufLen)
 {
     struct scsi_cmnd_io io_hdr;
@@ -324,6 +328,7 @@ int scsiModeSense(int device, int pagenum, int pc, UINT8 *pBuf, int bufLen)
     io_hdr.cmnd_len = sizeof(cdb);
     io_hdr.sensep = sense;
     io_hdr.max_sense_len = sizeof(sense);
+    io_hdr.timeout = SCSI_TIMEOUT_DEFAULT;
 
     status = do_scsi_cmnd_io(device, &io_hdr, con->reportscsiioctl);
     if (0 == status) {
@@ -378,6 +383,7 @@ int scsiModeSelect(int device, int sp, UINT8 *pBuf, int bufLen)
     io_hdr.cmnd_len = sizeof(cdb);
     io_hdr.sensep = sense;
     io_hdr.max_sense_len = sizeof(sense);
+    io_hdr.timeout = SCSI_TIMEOUT_DEFAULT;
 
     status = do_scsi_cmnd_io(device, &io_hdr, con->reportscsiioctl);
     if (0 != status)
@@ -389,7 +395,7 @@ int scsiModeSelect(int device, int sp, UINT8 *pBuf, int bufLen)
 /* MODE SENSE (10 byte). Returns 0 if ok, 1 if NOT READY, 2 if command 
  * not supported (then MODE SENSE(6) might be supported), 3 if field in
  * command not supported or returns negated errno.  
- * SPC sections 7.10 and 8.4 */
+ * SPC sections 7.10 and 8.4 [mode subpage==0] */
 int scsiModeSense10(int device, int pagenum, int pc, UINT8 *pBuf, int bufLen)
 {
     struct scsi_cmnd_io io_hdr;
@@ -411,6 +417,7 @@ int scsiModeSense10(int device, int pagenum, int pc, UINT8 *pBuf, int bufLen)
     io_hdr.cmnd_len = sizeof(cdb);
     io_hdr.sensep = sense;
     io_hdr.max_sense_len = sizeof(sense);
+    io_hdr.timeout = SCSI_TIMEOUT_DEFAULT;
 
     status = do_scsi_cmnd_io(device, &io_hdr, con->reportscsiioctl);
     if (0 == status) {
@@ -466,6 +473,7 @@ int scsiModeSelect10(int device, int sp, UINT8 *pBuf, int bufLen)
     io_hdr.cmnd_len = sizeof(cdb);
     io_hdr.sensep = sense;
     io_hdr.max_sense_len = sizeof(sense);
+    io_hdr.timeout = SCSI_TIMEOUT_DEFAULT;
 
     status = do_scsi_cmnd_io(device, &io_hdr, con->reportscsiioctl);
     if (0 != status)
@@ -531,6 +539,7 @@ int scsiInquiryVpd(int device, int vpd_page, UINT8 *pBuf, int bufLen)
     io_hdr.cmnd_len = sizeof(cdb);
     io_hdr.sensep = sense;
     io_hdr.max_sense_len = sizeof(sense);
+    io_hdr.timeout = SCSI_TIMEOUT_DEFAULT;
 
     status = do_scsi_cmnd_io(device, &io_hdr, con->reportscsiioctl);
     if (0 != status)
@@ -615,8 +624,8 @@ int scsiSendDiagnostic(int device, int functioncode, UINT8 *pBuf, int bufLen)
     io_hdr.cmnd_len = sizeof(cdb);
     io_hdr.sensep = sense;
     io_hdr.max_sense_len = sizeof(sense);
-    io_hdr.timeout = 5 * 60 * 60;   /* five hours because a foreground 
-                    extended self tests can take 1 hour plus */
+    io_hdr.timeout = SCSI_TIMEOUT_SELF_TEST;
+    /* worst case is an extended foreground self test on a big disk */
     
     status = do_scsi_cmnd_io(device, &io_hdr, con->reportscsiioctl);
     if (0 != status)
@@ -651,6 +660,7 @@ int scsiReceiveDiagnostic(int device, int pcv, int pagenum, UINT8 *pBuf,
     io_hdr.cmnd_len = sizeof(cdb);
     io_hdr.sensep = sense;
     io_hdr.max_sense_len = sizeof(sense);
+    io_hdr.timeout = SCSI_TIMEOUT_DEFAULT;
 
     status = do_scsi_cmnd_io(device, &io_hdr, con->reportscsiioctl);
     if (0 != status)
@@ -677,6 +687,7 @@ static int _testunitready(int device, struct scsi_sense_disect * sinfo)
     io_hdr.cmnd_len = sizeof(cdb);
     io_hdr.sensep = sense;
     io_hdr.max_sense_len = sizeof(sense);
+    io_hdr.timeout = SCSI_TIMEOUT_DEFAULT;
 
     status = do_scsi_cmnd_io(device, &io_hdr, con->reportscsiioctl);
     if (0 != status)
@@ -1611,7 +1622,7 @@ void scsiDecodeErrCounterPage(unsigned char * resp,
     int k, j, num, pl, pc;
     unsigned char * ucp;
     unsigned char * xp;
-    unsigned long long * ullp;
+    uint64_t * ullp;
 
     memset(ecp, 0, sizeof(*ecp));
     num = (resp[2] << 8) | resp[3];
@@ -1903,8 +1914,11 @@ int scsiFetchTransportProtocol(int device, int modese_len)
             return -EINVAL;
     } 
     offset = scsiModePageOffset(buff, sizeof(buff), modese_len);
-    if ((offset >= 0) && (buff[offset + 1] > 1))
-        return (buff[offset + 2] & 0xf);
+    if ((offset >= 0) && (buff[offset + 1] > 1)) {
+        if ((0 == (buff[offset] & 0x40)) &&       /* SPF==0 */
+            (PROTOCOL_SPECIFIC_PORT_PAGE == (buff[offset] & 0x3f))) 
+                return (buff[offset + 2] & 0xf);
+    }
     return -EINVAL;
 }
 
