@@ -32,14 +32,14 @@
 #ifndef SCSICMDS_H_
 #define SCSICMDS_H_
 
-#define SCSICMDS_H_CVSID "$Id: scsicmds.h,v 1.46 2004/01/02 16:05:25 ballen4705 Exp $\n"
+#define SCSICMDS_H_CVSID "$Id: scsicmds.h,v 1.48 2004/03/20 08:52:40 dpgilbert Exp $\n"
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <unistd.h>
 #include <errno.h>
-#include <sys/ioctl.h>
+
+#include "int64.h"
 
 /* #define SCSI_DEBUG 1 */ /* Comment out to disable command debugging */
 
@@ -127,14 +127,14 @@ struct scsi_iec_mode_page {
 struct scsiErrorCounter {
     UINT8 gotPC[7];
     UINT8 gotExtraPC;
-    unsigned long long counter[8];
+    uint64_t counter[8];
 };
 
 /* Carrier for Non-medium error log page */
 struct scsiNonMediumError {
     UINT8 gotPC0;
     UINT8 gotExtraPC;
-    unsigned long long counterPC0;
+    uint64_t counterPC0;
 };
 
 /* SCSI Peripheral types (of interest) */
@@ -232,7 +232,6 @@ Documentation, see http://www.storage.ibm.com/techsup/hddtech/prodspecs.htm */
 
 
 /* defines for functioncode parameter in SENDDIAGNOSTIC function */
-
 #define SCSI_DIAG_NO_SELF_TEST          0x00
 #define SCSI_DIAG_DEF_SELF_TEST         0xff
 #define SCSI_DIAG_BG_SHORT_SELF_TEST    0x01
@@ -242,6 +241,12 @@ Documentation, see http://www.storage.ibm.com/techsup/hddtech/prodspecs.htm */
 #define SCSI_DIAG_ABORT_SELF_TEST       0x04
 
 
+/* SCSI command timeout values (units are seconds) */
+#define SCSI_TIMEOUT_DEFAULT    6               /* 6 seconds should be ample */
+#define SCSI_TIMEOUT_SELF_TEST  (5 * 60 * 60)   /* allow max 5 hours for */
+                                            /* extended foreground self test */
+
+
 
 #define LOGPAGEHDRSIZE  4
 
