@@ -27,11 +27,12 @@
 
 
 #include <stdio.h>
-#include <unistd.h>
 #include <string.h>
 #include <fcntl.h>
 #include <errno.h>
 
+#include "config.h"
+#include "int64.h"
 #include "extern.h"
 #include "scsicmds.h"
 #include "scsiprint.h"
@@ -40,8 +41,8 @@
 
 #define GBUF_SIZE 65535
 
-const char* scsiprint_c_cvsid="$Id: scsiprint.c,v 1.73 2004/01/27 15:29:16 ballen4705 Exp $"
-EXTERN_H_CVSID SCSICMDS_H_CVSID SCSIPRINT_H_CVSID SMARTCTL_H_CVSID UTILITY_H_CVSID;
+const char* scsiprint_c_cvsid="$Id: scsiprint.c,v 1.78 2004/04/30 06:17:52 dpgilbert Exp $"
+CONFIG_H_CVSID EXTERN_H_CVSID INT64_H_CVSID SCSICMDS_H_CVSID SCSIPRINT_H_CVSID SMARTCTL_H_CVSID UTILITY_H_CVSID;
 
 // control block which points to external global control variables
 extern smartmonctrl *con;
@@ -114,7 +115,7 @@ static void scsiGetSupportedLogPages(int device)
     }
 }
 
-void scsiGetSmartData(int device)
+void scsiGetSmartData(int device, int attribs)
 {
     UINT8 asc;
     UINT8 ascq;
@@ -139,14 +140,18 @@ void scsiGetSmartData(int device)
     } else if (gIecMPage)
         pout("SMART Health Status: OK\n");
 
-    if (currenttemp && !gTempLPage) {
-        if (255 != currenttemp)
-            pout("Current Drive Temperature:     %d C\n", currenttemp);
-        else
-            pout("Current Drive Temperature:     <not available>\n");
+    if (attribs && !gTempLPage) {
+        if (currenttemp || triptemp)
+            pout("\n");
+        if (currenttemp) {
+            if (255 != currenttemp)
+                pout("Current Drive Temperature:     %d C\n", currenttemp);
+            else
+                pout("Current Drive Temperature:     <not available>\n");
+        }
+        if (triptemp)
+            pout("Drive Trip Temperature:        %d C\n", triptemp);
     }
-    if (triptemp && !gTempLPage)
-        pout("Drive Trip Temperature:        %d C\n", triptemp);
 }
 
 
@@ -249,7 +254,7 @@ static void scsiPrintSeagateCacheLPage(int device)
     int k, j, num, pl, pc, err, len;
     unsigned char * ucp;
     unsigned char * xp;
-    unsigned long long ull;
+    uint64_t ull;
 
     if ((err = scsiLogSense(device, SEAGATE_CACHE_LPAGE, gBuf,
                             LOG_RESP_LEN, 0))) {
@@ -276,7 +281,7 @@ static void scsiPrintSeagateCacheLPage(int device)
         default: 
             if (con->reportscsiioctl > 0) {
                 PRINT_ON(con);
-                pout("\nVendor (Seagate) cache lpage has unexpected parameter"
+                pout("Vendor (Seagate) cache lpage has unexpected parameter"
                      ", skip\n");
                 PRINT_OFF(con);
             }
@@ -285,7 +290,7 @@ static void scsiPrintSeagateCacheLPage(int device)
         num -= pl;
         ucp += pl;
     }
-    pout("\nVendor (Seagate) cache information\n");
+    pout("Vendor (Seagate) cache information\n");
     num = len - 4;
     ucp = &gBuf[0] + 4;
     while (num > 3) {
@@ -313,7 +318,7 @@ static void scsiPrintSeagateCacheLPage(int device)
                 ull <<= 8;
             ull |= xp[j];
         }
-        pout(" = %llu\n", ull);
+        pout(" = %"PRIu64"\n", ull);
         num -= pl;
         ucp += pl;
     }
@@ -324,7 +329,7 @@ static void scsiPrintSeagateFactoryLPage(int device)
     int k, j, num, pl, pc, len, err;
     unsigned char * ucp;
     unsigned char * xp;
-    unsigned long long ull;
+    uint64_t ull;
 
     if ((err = scsiLogSense(device, SEAGATE_FACTORY_LPAGE, gBuf,
                             LOG_RESP_LEN, 0))) {
@@ -385,9 +390,9 @@ static void scsiPrintSeagateFactoryLPage(int device)
             ull |= xp[j];
         }
         if (0 == pc)
-            pout(" = %.2f\n", ((double)ull) / 60.0 );
+            pout(" = %.2f\n", uint64_to_double(ull) / 60.0 );
         else
-            pout(" = %llu\n", ull);
+            pout(" = %"PRIu64"\n", ull);
         num -= pl;
         ucp += pl;
     }
@@ -436,11 +441,11 @@ static void scsiPrintErrorCounterLog(int device)
             if (! found[k])
                 continue;
             ecp = &errCounterArr[k];
-            pout("%s%8llu %8llu  %8llu  %8llu   %8llu", 
+            pout("%s%8"PRIu64" %8"PRIu64"  %8"PRIu64"  %8"PRIu64"   %8"PRIu64, 
                  pageNames[k], ecp->counter[0], ecp->counter[1], 
                  ecp->counter[2], ecp->counter[3], ecp->counter[4]);
-            processed_gb = ecp->counter[5] / 1000000000.0;
-            pout("   %12.3f    %8llu\n", processed_gb, ecp->counter[6]);
+            processed_gb = uint64_to_double(ecp->counter[5]) / 1000000000.0;
+            pout("   %12.3f    %8"PRIu64"\n", processed_gb, ecp->counter[6]);
         }
     }
     else 
@@ -449,7 +454,7 @@ static void scsiPrintErrorCounterLog(int device)
                           LOG_RESP_LEN, 0)) {
         scsiDecodeNonMediumErrPage(gBuf, &nme);
         if (nme.gotPC0)
-            pout("\nNon-medium error count: %8llu\n", nme.counterPC0);
+            pout("\nNon-medium error count: %8"PRIu64"\n", nme.counterPC0);
     }
 }
 
@@ -490,7 +495,7 @@ static int scsiPrintSelfTest(int device)
     int num, k, n, res, err, durationSec;
     int noheader = 1;
     UINT8 * ucp;
-    unsigned long long ull=0;
+    uint64_t ull=0;
 
     if ((err = scsiLogSense(device, SELFTEST_RESULTS_LPAGE, gBuf, 
                             LOG_RESP_SELF_TEST_LEN, 0))) {
@@ -568,8 +573,8 @@ static int scsiPrintSelfTest(int device)
             ull |= ucp[i+8];
         }
         // print Address of First Failure, if sensible
-        if ((0xffffffffffffffffULL != ull) && (res > 0) && ( res < 0xf))
-            pout("  0x%16llx", ull);
+        if ((~(uint64_t)0 != ull) && (res > 0) && (res < 0xf))
+            pout("  0x%16"PRIx64, ull);
         else
             pout("                   -");
 
@@ -638,7 +643,7 @@ static int scsiGetDriveInfo(int device, UINT8 * peripheral_type, int all)
     char manufacturer[9];
     char product[17];
     char revision[5];
-    char timedatetz[64];
+    char timedatetz[DATEANDEPOCHLEN];
     struct scsi_iec_mode_page iec;
     int err, iec_err, len, val;
     int is_tape = 0;
@@ -907,23 +912,26 @@ int scsiPrintMain(int fd)
             }
             else
                 pout("TapeAlert Not Supported\n");
-            if (gTempLPage)
-                scsiPrintTemp(fd);         
-            if (gStartStopLPage)
-                scsiGetStartStopData(fd);
         } else { /* disk, cd/dvd, enclosure, etc */
-            scsiGetSmartData(fd);
-            if (gTempLPage)
-                scsiPrintTemp(fd);         
-            if (gStartStopLPage)
-                scsiGetStartStopData(fd);
+            scsiGetSmartData(fd, con->smartvendorattrib);
         }
     }   
     if (con->smartvendorattrib) {
-        if (gSeagateCacheLPage && (SCSI_PT_DIRECT_ACCESS == peripheral_type))
-            scsiPrintSeagateCacheLPage(fd);
-        if (gSeagateFactoryLPage && (SCSI_PT_DIRECT_ACCESS == peripheral_type))
-            scsiPrintSeagateFactoryLPage(fd);
+        if (! checkedSupportedLogPages)
+            scsiGetSupportedLogPages(fd);
+        if (gTempLPage) {
+            if (con->checksmart)
+                pout("\n");
+            scsiPrintTemp(fd);         
+        }
+        if (gStartStopLPage)
+            scsiGetStartStopData(fd);
+        if (SCSI_PT_DIRECT_ACCESS == peripheral_type) {
+            if (gSeagateCacheLPage)
+                scsiPrintSeagateCacheLPage(fd);
+            if (gSeagateFactoryLPage)
+                scsiPrintSeagateFactoryLPage(fd);
+        }
     }
     if (con->smarterrorlog) {
         scsiPrintErrorCounterLog(fd);
