@@ -23,10 +23,8 @@
  */
 
 #include "config.h"
+#include <errno.h>
 #include <stdio.h>
-#include <unistd.h>
-#include <sys/ioctl.h>
-#include <sys/fcntl.h>
 #include <sys/types.h>
 #include <string.h>
 #include <stdarg.h>
@@ -36,6 +34,7 @@
 #include "atacmds.h"
 #include "ataprint.h"
 #include "extern.h"
+#include "int64.h"
 #include "knowndrives.h"
 #include "scsicmds.h"
 #include "scsiprint.h"
@@ -45,9 +44,12 @@
 #ifdef NEED_SOLARIS_ATA_CODE
 extern const char *os_solaris_ata_s_cvsid;
 #endif
-extern const char *atacmdnames_c_cvsid, *atacmds_c_cvsid, *ataprint_c_cvsid, *knowndrives_c_cvsid, *os_XXXX_c_cvsid, *scsicmds_c_cvsid, *scsiprint_c_cvsid, *utility_c_cvsid; 
-const char* smartctl_c_cvsid="$Id: smartctl.c,v 1.115 2004/02/13 17:29:09 ballen4705 Exp $"
-ATACMDS_H_CVSID ATAPRINT_H_CVSID CONFIG_H_CVSID EXTERN_H_CVSID KNOWNDRIVES_H_CVSID SCSICMDS_H_CVSID SCSIPRINT_H_CVSID SMARTCTL_H_CVSID UTILITY_H_CVSID;
+#if defined(_WIN32) && defined(_MSC_VER)
+extern const char *int64_vc6_c_cvsid;
+#endif
+extern const char *atacmdnames_c_cvsid, *atacmds_c_cvsid, *ataprint_c_cvsid, *knowndrives_c_cvsid, *os_XXXX_c_cvsid, *scsicmds_c_cvsid, *scsiprint_c_cvsid, *utility_c_cvsid;
+const char* smartctl_c_cvsid="$Id: smartctl.c,v 1.125 2004/04/30 06:23:59 dpgilbert Exp $"
+ATACMDS_H_CVSID ATAPRINT_H_CVSID CONFIG_H_CVSID EXTERN_H_CVSID INT64_H_CVSID KNOWNDRIVES_H_CVSID SCSICMDS_H_CVSID SCSIPRINT_H_CVSID SMARTCTL_H_CVSID UTILITY_H_CVSID;
 
 // This is a block containing all the "control variables".  We declare
 // this globally in this file, and externally in other files.
@@ -57,7 +59,7 @@ smartmonctrl *con=NULL;
 extern int exitstatus;
 
 // Track memory use
-extern long long bytes;
+extern int64_t bytes;
 
 void printslogan(){
   pout("smartctl version %s Copyright (C) 2002-4 Bruce Allen\n", PACKAGE_VERSION);
@@ -80,7 +82,11 @@ void printcopy(){
   pout("%s",out);
   printone(out,ataprint_c_cvsid);
   pout("%s",out);
-  printone(out,knowndrives_c_cvsid);
+ #if defined(_WIN32) && defined(_MSC_VER)
+  printone(out,int64_vc6_c_cvsid);
+  pout("%s",out);
+#endif
+ printone(out,knowndrives_c_cvsid);
   pout("%s",out);
   printone(out,os_XXXX_c_cvsid);
   pout("%s",out);
@@ -179,10 +185,10 @@ void Usage (void){
 "        Show device SMART health status\n\n"
 "  -c, --capabilities                                                  (ATA)\n"
 "        Show device SMART capabilities\n\n"
-"  -A, --attributes                                                    (ATA)\n"
+"  -A, --attributes                                                         \n"
 "        Show device SMART vendor-specific Attributes and values\n\n"
 "  -l TYPE, --log=TYPE\n"
-"        Show device log. Type is one of: error, selftest, directory\n\n"
+"        Show device log. TYPE: error, selftest, selective, directory\n\n"
 "  -v N,OPTION , --vendorattribute=N,OPTION                            (ATA)\n"
 "        Set display OPTION for vendor Attribute N (see man page)\n\n"
 "  -F TYPE, --firmwarebug=TYPE                                         (ATA)\n"
@@ -195,7 +201,7 @@ void Usage (void){
 "  -H        Show device SMART health status\n"
 "  -c        Show device SMART capabilities                             (ATA)\n"
 "  -A        Show device SMART vendor-specific Attributes and values    (ATA)\n"
-"  -l TYPE   Show device log. Type is one of: error, selftest, directory\n"
+"  -l TYPE   Show device log. TYPE: error,selftest,selective,directory\n"
 "  -v N,OPT  Set display OPTion for vendor Attribute N (see man page)   (ATA)\n"
 "  -F TYPE   Use firmware bug workaround: none, samsung, samsung2       (ATA)\n"
 "  -P TYPE   Drive-specific presets: use, ignore, show, showall         (ATA)\n\n"
@@ -205,31 +211,16 @@ void Usage (void){
 #ifdef HAVE_GETOPT_LONG
   printf(
 "  -t TEST, --test=TEST\n"
-#if DEVELOP_SELECTIVE_SELF_TEST
-"        Run test.  TEST is: offline, short, long, conveyance, selective,M-N\n\n"
-#else
-"        Run test.  TEST is: offline, short, long, conveyance\n\n"
-#endif
-
+"        Run test.  TEST is: offline short long conveyance select,M-N pending,N afterselect,on afterselect,off\n\n"
 "  -C, --captive\n"
-#if DEVELOP_SELECTIVE_SELF_TEST
-"        With -t, do test in captive mode (short/long/conveyance/selective)\n\n"
-#else
-"        With -t, do test in captive mode (short/long/conveyance)\n\n"
-#endif
-
+"        Do test in captive mode (along with -t)\n\n"
 "  -X, --abort\n"
 "        Abort any non-captive test on device\n\n"
 );
 #else
   printf(
-#if DEVELOP_SELECTIVE_SELF_TEST
-"  -t TEST   Run test.  TEST is: offline, short, long, conveyance, selective,M-N\n"
-"  -C        With -t, do test in captive mode (short/long/conveyance/selective)\n"
-#else
-"  -t TEST   Run test.  TEST is: offline, short, long, conveyance\n"
-"  -C        With -t, do test in captive mode (short/long/conveyance)\n"
-#endif
+"  -t TEST   Run test.  TEST is: offline short long conveyance select,M-N pending,N afterselect,on afterselect,off\n"
+"  -C        Do test in captive mode (along with -t)\n"
 "  -X        Abort any non-captive test\n\n"
   );
 #endif
@@ -256,15 +247,11 @@ const char *getvalidarglist(char opt) {
   case 'S':
     return "on, off";
   case 'l':
-    return "error, selftest, directory";
+    return "error, selftest, selective, directory";
   case 'P':
     return "use, ignore, show, showall";
   case 't':
-#if DEVELOP_SELECTIVE_SELF_TEST
-    return "offline, short, long, conveyance, selective,M-N";
-#else
-    return "offline, short, long, conveyance";
-#endif
+    return "offline, short, long, conveyance, select,M-N, pending,N, afterselect,on, afterselect,off";
   case 'F':
     return "none, samsung, samsung2";
   case 'v':
@@ -273,7 +260,7 @@ const char *getvalidarglist(char opt) {
   }
 }
 
-/* Prints the message "=======> VALID ARGUMENTS ARE: <LIST>  <=======\n", where
+/* Prints the message "=======> VALID ARGUMENTS ARE: <LIST> \n", where
    <LIST> is the list of valid arguments for option opt. */
 void printvalidarglistmessage(char opt) {
   char *s;
@@ -313,6 +300,7 @@ void ParseOpts (int argc, char** argv){
   unsigned char *charp;
   extern char *optarg;
   extern int optopt, optind, opterr;
+  char extraerror[256];
   // Please update getvalidarglist() if you edit shortopts
   const char *shortopts = "h?Vq:d:T:b:r:s:o:S:HcAl:iav:P:t:CXF:";
 #ifdef HAVE_GETOPT_LONG
@@ -348,6 +336,7 @@ void ParseOpts (int argc, char** argv){
   };
 #endif
   
+  memset(extraerror, 0, sizeof(extraerror));
   memset(con,0,sizeof(*con));
   con->testcase=-1;
   opterr=optopt=0;
@@ -401,10 +390,10 @@ void ParseOpts (int argc, char** argv){
         } else if (strncmp(s,"3ware,",6)) {
           badarg = TRUE;
         } else if (split_report_arg2(s, &i)) {
-          pout("Option -d 3ware,N requires N to be a non-negative integer\n");
+          sprintf(extraerror, "Option -d 3ware,N requires N to be a non-negative integer\n");
           badarg = TRUE;
         } else if (i<0 || i>15) {
-          pout("Option -d 3ware,N (N=%d) must have 0 <= N <= 15\n", i);
+          sprintf(extraerror, "Option -d 3ware,N (N=%d) must have 0 <= N <= 15\n", i);
           badarg = TRUE;
         } else {
           // NOTE: escalade = disk number + 1
@@ -529,6 +518,8 @@ void ParseOpts (int argc, char** argv){
         con->smarterrorlog = TRUE;
       } else if (!strcmp(optarg,"selftest")) {
         con->smartselftestlog = TRUE;
+      } else if (!strcmp(optarg, "selective")) {
+	con->selectivetestlog = TRUE;
       } else if (!strcmp(optarg,"directory")) {
         con->smartlogdirectory = TRUE;
       } else {
@@ -545,6 +536,7 @@ void ParseOpts (int argc, char** argv){
       con->smartvendorattrib  = TRUE;
       con->smarterrorlog      = TRUE;
       con->smartselftestlog   = TRUE;
+      con->selectivetestlog   = TRUE;
       break;
     case 'v':
       // parse vendor-specific definitions of attributes
@@ -595,32 +587,50 @@ void ParseOpts (int argc, char** argv){
       } else if (!strcmp(optarg,"conveyance")) {
         con->smartconveyanceselftest = TRUE;
         con->testcase            = CONVEYANCE_SELF_TEST;
-#if DEVELOP_SELECTIVE_SELF_TEST
-      } else if (!strncmp(optarg,"selective",strlen("selective"))) {
-        unsigned long long start, stop;
+      } else if (!strcmp(optarg,"afterselect,on")) {
+	// scan remainder of disk after doing selected segments
+	con->scanafterselect=2;
+      } else if (!strcmp(optarg,"afterselect,off")) {
+	// don't scan remainder of disk after doing selected segments
+	con->scanafterselect=1;
+      } else if (!strncmp(optarg,"pending,",strlen("pending,"))) {
+	// parse number of minutes that test should be pending
+	int i;
+	char *tailptr=NULL;
+	errno=0;
+	i=(int)strtol(optarg+strlen("pending,"), &tailptr, 10);
+	if (errno || *tailptr != '\0') {
+	  sprintf(extraerror, "Option -t pending,N requires N to be a non-negative integer\n");
+	  badarg = TRUE;
+	} else if (i<0 || i>65535) {
+	  sprintf(extraerror, "Option -t pending,N (N=%d) must have 0 <= N <= 65535\n", i);
+	  badarg = TRUE;
+	} else {
+	  con->pendingtime=i+1;
+	}
+      } else if (!strncmp(optarg,"select",strlen("select"))) {
+	// parse range of LBAs to test
+	uint64_t start, stop;
 
         if (split_selective_arg(optarg, &start, &stop)) {
+	  sprintf(extraerror, "Option -t select,M-N must have non-negative integer M and N\n");
           badarg = TRUE;
         } else {
           if (con->smartselectivenumspans >= 5 || start > stop) {
-            con->dont_print=FALSE;
-            printslogan();
             if (start > stop) {
-              pout("ERROR: Start LBA > ending LBA in argument \"%s\"\n",
-                optarg);
+              sprintf(extraerror, "ERROR: Start LBA (%"PRIu64") > ending LBA (%"PRId64") in argument \"%s\"\n",
+                start, stop, optarg);
             } else {
-              pout("ERROR: No more than five selective self-test spans may be"
+              sprintf(extraerror,"ERROR: No more than five selective self-test spans may be"
                 " defined\n");
             }
-            UsageSummary();
-            EXIT(FAILCMD);
+	    badarg = TRUE;
           }
           con->smartselectivespan[con->smartselectivenumspans][0] = start;
           con->smartselectivespan[con->smartselectivenumspans][1] = stop;
           con->smartselectivenumspans++;
           con->testcase            = SELECTIVE_SELF_TEST;
         }
-#endif
       } else {
         badarg = TRUE;
       }
@@ -649,10 +659,12 @@ void ParseOpts (int argc, char** argv){
       if (arg[1] == '-' && optchar != 'h') {
         // Iff optopt holds a valid option then argument must be missing.
         if (optopt && (strchr(shortopts, optopt) != NULL)) {
-          pout("=======> ARGUMENT REQUIRED FOR OPTION: %s <=======\n", arg+2);
+          pout("=======> ARGUMENT REQUIRED FOR OPTION: %s\n", arg+2);
           printvalidarglistmessage(optopt);
         } else
-          pout("=======> UNRECOGNIZED OPTION: %s <=======\n",arg+2);
+          pout("=======> UNRECOGNIZED OPTION: %s\n",arg+2);
+	if (extraerror[0])
+	  pout("=======> %s", extraerror);
         UsageSummary();
         EXIT(FAILCMD);
       }
@@ -662,10 +674,12 @@ void ParseOpts (int argc, char** argv){
         // missing.  Note (BA) this logic seems to fail using Solaris
         // getopt!
         if (strchr(shortopts, optopt) != NULL) {
-          pout("=======> ARGUMENT REQUIRED FOR OPTION: %c <=======\n", optopt);
+          pout("=======> ARGUMENT REQUIRED FOR OPTION: %c\n", optopt);
           printvalidarglistmessage(optopt);
         } else
-          pout("=======> UNRECOGNIZED OPTION: %c <=======\n",optopt);
+          pout("=======> UNRECOGNIZED OPTION: %c\n",optopt);
+	if (extraerror[0])
+	  pout("=======> %s", extraerror);
         UsageSummary();
         EXIT(FAILCMD);
       }
@@ -679,8 +693,10 @@ void ParseOpts (int argc, char** argv){
       // It would be nice to print the actual option name given by the user
       // here, but we just print the short form.  Please fix this if you know
       // a clean way to do it.
-      pout("=======> INVALID ARGUMENT TO -%c: %s <======= \n", optchar, optarg);
+      pout("=======> INVALID ARGUMENT TO -%c: %s\n", optchar, optarg);
       printvalidarglistmessage(optchar);
+      if (extraerror[0])
+	pout("=======> %s", extraerror);
       UsageSummary();
       EXIT(FAILCMD);
     }
@@ -693,10 +709,23 @@ void ParseOpts (int argc, char** argv){
 
   // error message if user has asked for more than one test
   if (1<(con->smartexeoffimmediate+con->smartshortselftest+con->smartextendselftest+
-         con->smartshortcapselftest+con->smartextendcapselftest+con->smartselftestabort)){
+         con->smartshortcapselftest+con->smartextendcapselftest+con->smartselftestabort + (con->smartselectivenumspans>0?1:0))){
     con->dont_print=FALSE;
     printslogan();
-    pout("\nERROR: smartctl can only run a single test (or abort) at a time.\n");
+    pout("\nERROR: smartctl can only run a single test type (or abort) at a time.\n");
+    UsageSummary();
+    EXIT(FAILCMD);
+  }
+
+  // error message if user has set selective self-test options without
+  // asking for a selective self-test
+  if ((con->pendingtime || con->scanafterselect) && !con->smartselectivenumspans){
+    con->dont_print=FALSE;
+    printslogan();
+    if (con->pendingtime)
+      pout("\nERROR: smartctl -t pending,N must be used with -t select,N-M.\n");
+    else
+      pout("\nERROR: smartctl -t afterselect,(on|off) must be used with -t select,N-M.\n");
     UsageSummary();
     EXIT(FAILCMD);
   }
@@ -716,13 +745,11 @@ void ParseOpts (int argc, char** argv){
     con->smartconveyancecapselftest = TRUE;
     con->testcase                   = CONVEYANCE_CAPTIVE_SELF_TEST;
   }
-#if DEVELOP_SELECTIVE_SELF_TEST
   else if (captive && con->smartselectiveselftest) {
     con->smartselectiveselftest    = FALSE;
     con->smartselectivecapselftest = TRUE;
     con->testcase                  = SELECTIVE_CAPTIVE_SELF_TEST;
   }
-#endif 
  
   // From here on, normal operations...
   printslogan();
@@ -770,6 +797,21 @@ void pout(char *fmt, ...){
   return;
 }
 
+// This function is used by utility.c to report LOG_CRIT errors.
+// The smartctl version prints to stdout instead of syslog().
+void PrintOut(int priority, char *fmt, ...) {
+  va_list ap;
+
+  // avoid warning message about unused variable from gcc -W: just
+  // change value of local copy.
+  priority=0;
+
+  va_start(ap,fmt);
+  vprintf(fmt,ap);
+  va_end(ap);
+  return;
+}
+
 
 /* Main Program */
 int main (int argc, char **argv){
