@@ -27,7 +27,6 @@
 // SMARTCTL, OR BOTH.
 
 #include <stdio.h>
-#include <unistd.h>
 #include <string.h>
 #include <time.h>
 #include <errno.h>
@@ -35,11 +34,15 @@
 #include <ctype.h>
 #include <syslog.h>
 #include <stdarg.h>
-#include "utility.h"
+#include <sys/stat.h>
+
 #include "config.h"
+#include "int64.h"
+#include "utility.h"
 
 // Any local header files should be represented by a CVSIDX just below.
-const char* utility_c_cvsid="$Id: utility.c,v 1.35 2004/02/13 17:29:10 ballen4705 Exp $" CONFIG_H_CVSID UTILITY_H_CVSID;
+const char* utility_c_cvsid="$Id: utility.c,v 1.51 2004/05/27 16:26:36 card_captor Exp $"
+CONFIG_H_CVSID INT64_H_CVSID UTILITY_H_CVSID;
 
 const char * packet_types[] = {
         "Direct-access (disk)",
@@ -68,14 +71,109 @@ const char *reportbug="Please report this bug to the Smartmontools developers at
 // functionality and still check our exit code
 int exitstatus = 0;
 
-// facility used by syslog(3).  smartctl should NEVER use this, and
-// smartd can reset it with a command-line argument
-int facility=LOG_DAEMON;
-
 // command-line argument: are we running in debug mode?.
 unsigned char debugmode = 0;
 
 
+// Solaris only: Get site-default timezone. This is called from
+// UpdateTimezone() when TZ environment variable is unset at startup.
+#if defined (__SVR4) && defined (__sun)
+static const char *TIMEZONE_FILE = "/etc/TIMEZONE";
+
+static char *ReadSiteDefaultTimezone(){
+  FILE *fp;
+  char buf[512], *tz;
+  int n;
+
+  tz = NULL;
+  fp = fopen(TIMEZONE_FILE, "r");
+  if(fp == NULL) return NULL;
+  while(fgets(buf, sizeof(buf), fp)) {
+    if (strncmp(buf, "TZ=", 3))    // searches last "TZ=" line
+      continue;
+    n = strlen(buf) - 1;
+    if (buf[n] == '\n') buf[n] = 0;
+    if (tz) free(tz);
+    tz = strdup(buf);
+  }
+  fclose(fp);
+  return tz;
+}
+#endif
+
+// Make sure that this executable is aware if the user has changed the
+// time-zone since the last time we polled devices. The cannonical
+// example is a user who starts smartd on a laptop, then flies across
+// time-zones with a laptop, and then changes the timezone, WITHOUT
+// restarting smartd. This is a work-around for a bug in
+// GLIBC. Yuk. See bug number 48184 at http://bugs.debian.org and
+// thanks to Ian Redfern for posting a workaround.
+
+// Please refer to the smartd manual page, in the section labeled LOG
+// TIMESTAMP TIMEZONE.
+void FixGlibcTimeZoneBug(){
+#if __GLIBC__  
+  if (!getenv("TZ")) {
+    putenv("TZ=GMT");
+    tzset();
+    putenv("TZ");
+    tzset();
+  }
+#elif _WIN32
+  if (!getenv("TZ")) {
+    putenv("TZ=GMT");
+    tzset();
+    putenv("TZ=");  // empty value removes TZ, putenv("TZ") does nothing
+    tzset();
+  }
+#elif defined (__SVR4) && defined (__sun)
+  // In Solaris, putenv("TZ=") sets null string and invalid timezone.
+  // putenv("TZ") does nothing.  With invalid TZ, tzset() do as if
+  // TZ=GMT.  With TZ unset, /etc/TIMEZONE will be read only _once_ at
+  // first tzset() call.  Conclusion: Unlike glibc, dynamic
+  // configuration of timezone can be done only by changing actual
+  // value of TZ environment value.
+  enum tzstate { NOT_CALLED_YET, USER_TIMEZONE, TRACK_TIMEZONE };
+  static enum tzstate state = NOT_CALLED_YET;
+
+  static struct stat prev_stat;
+  static char *prev_tz;
+  struct stat curr_stat;
+  char *curr_tz;
+
+  if(state == NOT_CALLED_YET) {
+    if(getenv("TZ")) {
+      state = USER_TIMEZONE; // use supplied timezone
+    } else {
+      state = TRACK_TIMEZONE;
+      if(stat(TIMEZONE_FILE, &prev_stat)) {
+	state = USER_TIMEZONE;	// no TZ, no timezone file; use GMT forever
+      } else {
+	prev_tz = ReadSiteDefaultTimezone(); // track timezone file change
+	if(prev_tz) putenv(prev_tz);
+      }
+    }
+    tzset();
+  } else if(state == TRACK_TIMEZONE) {
+    if(stat(TIMEZONE_FILE, &curr_stat) == 0
+       && (curr_stat.st_ctime != prev_stat.st_ctime
+	    || curr_stat.st_mtime != prev_stat.st_mtime)) {
+      // timezone file changed
+      curr_tz = ReadSiteDefaultTimezone();
+      if(curr_tz) {
+	putenv(curr_tz);
+	if(prev_tz) free(prev_tz);
+	prev_tz = curr_tz; prev_stat = curr_stat; 
+      }
+    }
+    tzset();
+  }
+#endif
+  // OTHER OS/LIBRARY FIXES SHOULD GO HERE, IF DESIRED.  PLEASE TRY TO
+  // KEEP THEM INDEPENDENT.
+  return;
+}
+
 // This value follows the peripheral device type value as defined in
 // SCSI Primary Commands, ANSI INCITS 301:1997.  It is also used in
 // the ATA standard for packet devices to define the device type.
@@ -100,15 +198,16 @@ int isbigendian(){
   return *tmp;
 }
 
-#define DBLEN 64
-
 // Utility function prints date and time and timezone into a character
 // buffer of length>=64.  All the fuss is needed to get the right
 // timezone info (sigh).
 void dateandtimezoneepoch(char *buffer, time_t tval){
   struct tm *tmval;
   char *timezonename;
-  char datebuffer[DBLEN];
+  char datebuffer[DATEANDEPOCHLEN];
+  int lenm1;
+
+  FixGlibcTimeZoneBug();
   
   // Get the time structure.  We need this to determine if we are in
   // daylight savings time or not.
@@ -116,11 +215,12 @@ void dateandtimezoneepoch(char *buffer, time_t tval){
   
   // Convert to an ASCII string, put in datebuffer
   // same as: asctime_r(tmval, datebuffer);
-  strncpy(datebuffer, asctime(tmval), DBLEN);
-  datebuffer[DBLEN-1]='\0';
+  strncpy(datebuffer, asctime(tmval), DATEANDEPOCHLEN);
+  datebuffer[DATEANDEPOCHLEN-1]='\0';
   
   // Remove newline
-  datebuffer[strlen(datebuffer)-1]='\0';
+  lenm1=strlen(datebuffer)-1;
+  datebuffer[lenm1>=0?lenm1:0]='\0';
   
   // correct timezone name
   if (tmval->tm_isdst==0)
@@ -134,7 +234,7 @@ void dateandtimezoneepoch(char *buffer, time_t tval){
     timezonename="";
   
   // Finally put the information into the buffer as needed.
-  snprintf(buffer, DBLEN, "%s %s", datebuffer, timezonename);
+  snprintf(buffer, DATEANDEPOCHLEN, "%s %s", datebuffer, timezonename);
   
   return;
 }
@@ -314,8 +414,8 @@ int split_report_arg2(char *s, int *i){
 // "selective,%lld-%lld" (prefixes of "0" (for octal) and "0x"/"0X" (for hex)
 // are allowed).  The first long long int is assigned to *start and the second
 // to *stop.  Returns zero if successful and non-zero otherwise.
-int split_selective_arg(char *s, unsigned long long *start,
-                        unsigned long long *stop)
+int split_selective_arg(char *s, uint64_t *start,
+                        uint64_t *stop)
 {
   char *tailptr;
 
@@ -327,6 +427,7 @@ int split_selective_arg(char *s, unsigned long long *start,
   // Last argument to strtoull (the base) is 0 meaning that decimal is assumed
   // unless prefixes of "0" (for octal) or "0x"/"0X" (for hex) are used.
   *start = strtoull(s, &tailptr, 0);
+
   s = tailptr;
   if (errno || *s++ != '-')
     return 1;
@@ -336,7 +437,7 @@ int split_selective_arg(char *s, unsigned long long *start,
   return 0;
 }
 
-long long bytes = 0;
+int64_t bytes = 0;
 // Helps debugging.  If the second argument is non-negative, then
 // decrement bytes by that amount.  Else decrement bytes by (one plus)
 // length of null terminated string.
@@ -374,9 +475,6 @@ void *Calloc(size_t nmemb, size_t size) {
   return ptr;
 }
 
-
-
-
 // A custom version of strdup() that keeps track of how much memory is
 // being allocated. If mustexist is set, it also throws an error if we
 // try to duplicate a NULL string.
@@ -408,30 +506,6 @@ char *CustomStrDup(char *ptr, int mustexist, int whatline, const char* file){
   return tmp;
 }
 
-// This function prints either to stdout or to the syslog as needed
-
-// [From GLIBC Manual: Since the prototype doesn't specify types for
-// optional arguments, in a call to a variadic function the default
-// argument promotions are performed on the optional argument
-// values. This means the objects of type char or short int (whether
-// signed or not) are promoted to either int or unsigned int, as
-// appropriate.]
-extern int facility;
-void PrintOut(int priority,char *fmt, ...){
-  va_list ap;
-  // initialize variable argument list 
-  va_start(ap,fmt);
-  if (debugmode) 
-    vprintf(fmt,ap);
-  else {
-    openlog("smartd",LOG_PID, facility);
-    vsyslog(priority,fmt,ap);
-    closelog();
-  }
-  va_end(ap);
-  return;
-}
-
 // Returns nonzero if region of memory contains non-zero entries
 int nonempty(unsigned char *testarea,int n){
   int i;
@@ -440,3 +514,35 @@ int nonempty(unsigned char *testarea,int n){
       return 1;
   return 0;
 }
+
+
+// This routine converts an integer number of milliseconds into a test
+// string of the form Xd+Yh+Zm+Ts.msec.  The resulting text string is
+// written to the array.
+void MsecToText(unsigned int msec, char *txt){
+  int start=0;
+  unsigned int days, hours, min, sec;
+
+  days       = msec/86400000U;
+  msec      -= days*86400000U;
+
+  hours      = msec/3600000U; 
+  msec      -= hours*3600000U;
+
+  min        = msec/60000U;
+  msec      -= min*60000U;
+
+  sec        = msec/1000U;
+  msec      -= sec*1000U;
+
+  if (days) {
+    txt += sprintf(txt, "%2dd+", (int)days);
+    start=1;
+  }
+
+  sprintf(txt, "%02d:%02d:%02d.%03d", (int)hours, (int)min, (int)sec, (int)msec);  
+  return;
+}
+
+
+
