@@ -25,14 +25,23 @@
 #ifndef UTILITY_H_
 #define UTILITY_H_
 
-#define UTILITY_H_CVSID "$Id: utility.h,v 1.27 2004/02/06 03:52:02 ballen4705 Exp $\n"
+#define UTILITY_H_CVSID "$Id: utility.h,v 1.33 2004/04/14 13:27:20 chrfranke Exp $\n"
 
 #include <time.h>
+#include <sys/types.h> // for regex.h (according to POSIX)
 #include <regex.h>
 
+#include "int64.h"
+
+#if defined(_WIN32) && defined(_MSC_VER)
+#define snprintf  _snprintf
+#define vsnprintf  _vsnprintf
+#endif
+
 // Utility function prints current date and time and timezone into a
 // character buffer of length>=64.  All the fuss is needed to get the
 // right timezone info (sigh).
+#define DATEANDEPOCHLEN 64
 void dateandtimezone(char *buffer);
 // Same, but for time defined by epoch tval
 void dateandtimezoneepoch(char *buffer, time_t tval);
@@ -66,8 +75,7 @@ int split_report_arg(char *s, int *i);
 // Function for processing -c option in smartctl and smartd
 int split_report_arg2(char *s, int *i);
 // Function for processing -t selective... option in smartctl
-int split_selective_arg(char *s, unsigned long long *start,
-                        unsigned long long *stop);
+int split_selective_arg(char *s, uint64_t *start, uint64_t *stop);
 
 // Guess device type (ata or scsi) based on device name 
 #define GUESS_DEVTYPE_ATA       0
@@ -122,11 +130,19 @@ int deviceclose(int fd);
 // returns 1 if any of the n bytes are nonzero, else zero.
 int nonempty(unsigned char *testarea,int n);
 
+// needed to fix glibc bug
+void FixGlibcTimeZoneBug();
+
+// convert time in msec to a text string
+void MsecToText(unsigned int msec, char *txt);
+
 // Exit codes
 #define EXIT_BADCMD    1   // command line did not parse
-#define EXIT_BADCONF   2   // problem reading/parsing config file
+#define EXIT_BADCONF   2   // syntax error in config file
 #define EXIT_STARTUP   3   // problem forking daemon
 #define EXIT_PID       4   // problem creating pid file
+#define EXIT_NOCONF    5   // config file does not exist
+#define EXIT_READCONF  6   // config file exists but cannot be read
 
 #define EXIT_NOMEM     8   // out of memory
 #define EXIT_BADCODE   10  // internal error - should NEVER happen
